/*
 * AutomatonAccess.cpp
 *
 *  Created on: 20. 9. 2014
 *	  Author: Jan Travnicek
 */

#include "AutomatonAccess.h"

#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

#include <automaton/PDA/InputDrivenNPDA.h>
#include <automaton/PDA/InputDrivenDPDA.h>

#include <automaton/PDA/DPDA.h>
#include <automaton/PDA/NPDA.h>

#include <factory/XmlDataFactory.hpp>
#include <container/ObjectsSet.h>
#include <container/ObjectsMap.h>
#include <container/ObjectsTuple.h>
#include <container/ObjectsVector.h>
#include <container/ObjectsVariant.h>
#include <container/ObjectsPair.h>
#include <container/ObjectsTree.h>

#include "common/AccessHelpers.hpp"

void AutomatonAccess::access ( automaton::Automaton & automaton, const AutomatonSettings::Settings & settings, const OperationSettings::Settings & operation, std::deque < sax::Token > & argument ) {
	dispatch ( automaton.getData ( ), settings, operation, argument );
}

void AutomatonAccess::access ( automaton::NFA < > & automaton, const AutomatonSettings::Settings & settings, const OperationSettings::Settings & operation, std::deque < sax::Token > & argument ) {
	if ( settings == AutomatonSettings::Settings::STATES )
		return handleComponent < automaton::States > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::FINAL_STATES )
		return handleComponent < automaton::FinalStates > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::INITIAL_STATES )
		return handleElement < automaton::InitialState > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::INPUT_ALPHABET )
		return handleComponent < automaton::InputAlphabet > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::TRANSITIONS && operation == OperationSettings::Settings::GET )
		return alib::XmlDataFactory::toStdout ( automaton.getTransitions ( ) );

	throw exception::CommonException ( "Component not available" );
}

auto AutomatonAccessNFA = AutomatonAccess::RegistratorWrapper < void, automaton::NFA < > > ( AutomatonAccess::access );

void AutomatonAccess::access ( automaton::DFA<> & automaton, const AutomatonSettings::Settings & settings, const OperationSettings::Settings & operation, std::deque < sax::Token > & argument ) {
	if ( settings == AutomatonSettings::Settings::STATES )
		return handleComponent < automaton::States > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::FINAL_STATES )
		return handleComponent < automaton::FinalStates > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::INITIAL_STATES )
		return handleElement < automaton::InitialState > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::INPUT_ALPHABET )
		return handleComponent < automaton::InputAlphabet > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::TRANSITIONS && operation == OperationSettings::Settings::GET )
		return alib::XmlDataFactory::toStdout ( automaton.getTransitions ( ) );

	throw exception::CommonException ( "Component not available" );
}

auto AutomatonAccessDFA = AutomatonAccess::RegistratorWrapper < void, automaton::DFA<> > ( AutomatonAccess::access );

void AutomatonAccess::access ( automaton::InputDrivenNPDA < > & automaton, const AutomatonSettings::Settings & settings, const OperationSettings::Settings & operation, std::deque < sax::Token > & argument ) {
	if ( settings == AutomatonSettings::Settings::STATES )
		return handleComponent < automaton::States > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::FINAL_STATES )
		return handleComponent < automaton::FinalStates > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::INITIAL_STATES )
		return handleElement < automaton::InitialState > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::INPUT_ALPHABET )
		return handleComponent < automaton::InputAlphabet > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::TRANSITIONS && operation == OperationSettings::Settings::GET )
		return alib::XmlDataFactory::toStdout ( automaton.getTransitions ( ) );

	throw exception::CommonException ( "Component not available" );
}

auto AutomatonAccessInputDrivenNPDA = AutomatonAccess::RegistratorWrapper < void, automaton::InputDrivenNPDA < > > ( AutomatonAccess::access );

void AutomatonAccess::access ( automaton::InputDrivenDPDA < > & automaton, const AutomatonSettings::Settings & settings, const OperationSettings::Settings & operation, std::deque < sax::Token > & argument ) {
	if ( settings == AutomatonSettings::Settings::STATES )
		return handleComponent < automaton::States > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::FINAL_STATES )
		return handleComponent < automaton::FinalStates > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::INITIAL_STATES )
		return handleElement < automaton::InitialState > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::INPUT_ALPHABET )
		return handleComponent < automaton::InputAlphabet > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::TRANSITIONS && operation == OperationSettings::Settings::GET )
		return alib::XmlDataFactory::toStdout ( automaton.getTransitions ( ) );

	throw exception::CommonException ( "Component not available" );
}

auto AutomatonAccessInputDrivenDPDA = AutomatonAccess::RegistratorWrapper < void, automaton::InputDrivenDPDA < > > ( AutomatonAccess::access );

void AutomatonAccess::access ( automaton::NPDA < > & automaton, const AutomatonSettings::Settings & settings, const OperationSettings::Settings & operation, std::deque < sax::Token > & argument ) {
	if ( settings == AutomatonSettings::Settings::STATES )
		return handleComponent < automaton::States > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::FINAL_STATES )
		return handleComponent < automaton::FinalStates > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::INITIAL_STATES )
		return handleElement < automaton::InitialState > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::INPUT_ALPHABET )
		return handleComponent < automaton::InputAlphabet > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::TRANSITIONS && operation == OperationSettings::Settings::GET )
		return alib::XmlDataFactory::toStdout ( automaton.getTransitions ( ) );

	throw exception::CommonException ( "Component not available" );
}

auto AutomatonAccessNPDA = AutomatonAccess::RegistratorWrapper < void, automaton::NPDA < > > ( AutomatonAccess::access );

void AutomatonAccess::access ( automaton::DPDA < > & automaton, const AutomatonSettings::Settings & settings, const OperationSettings::Settings & operation, std::deque < sax::Token > & argument ) {
	if ( settings == AutomatonSettings::Settings::STATES )
		return handleComponent < automaton::States > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::FINAL_STATES )
		return handleComponent < automaton::FinalStates > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::INITIAL_STATES )
		return handleElement < automaton::InitialState > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::INPUT_ALPHABET )
		return handleComponent < automaton::InputAlphabet > ( automaton, operation, argument );

	if ( settings == AutomatonSettings::Settings::TRANSITIONS && operation == OperationSettings::Settings::GET )
		return alib::XmlDataFactory::toStdout ( automaton.getTransitions ( ) );

	throw exception::CommonException ( "Component not available" );
}

auto AutomatonAccessDPDA = AutomatonAccess::RegistratorWrapper < void, automaton::DPDA < > > ( AutomatonAccess::access );
