/*
 * aderivation.cpp
 *
 *  Created on: 19. 1. 2014
 *      Author: Tomas Pecka
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <sax/FromXMLParserHelper.h>

#include <factory/XmlDataFactory.hpp>
#include <exception/CommonException.h>

#include "regexp/transform/RegExpDerivation.h"

int main(int argc, char** argv) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd("Regexp derivative compute binary", ' ', "0.01");
		TCLAP::ValueArg<std::string> string(	"s",	"string",	"String to derivate by",	false,	"-",		"file");
		cmd.add( string );

		TCLAP::ValueArg<std::string> regexp(	"r",	"regexp",	"Regexp to derivate",		false,	"-",		"file");
		cmd.add( regexp );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc, argv);

		if(verbose.isSet())
			common::GlobalData::verbose = true;
		if(measure.isSet())
			common::GlobalData::measure = true;

		measurements::start("Overal", measurements::Type::OVERALL);
		measurements::start("Input read", measurements::Type::AUXILIARY);

		string::LinearString < > stringData = alib::XmlDataFactory::fromTokens<string::LinearString < >>(sax::FromXMLParserHelper::parseInput(string));
		regexp::RegExp regexpData = alib::XmlDataFactory::fromTokens<regexp::RegExp>(sax::FromXMLParserHelper::parseInput(regexp));

		measurements::end();
		measurements::start("Algorithm", measurements::Type::MAIN);

		regexp::RegExp result = regexp::RegExpDerivation::derivation(regexpData, stringData);

		measurements::end();
		measurements::start("Output write", measurements::Type::AUXILIARY);

		alib::XmlDataFactory::toStdout(result);

		measurements::end();
		measurements::end();

		if(measure.getValue()) std::cmeasure << measurements::results() << std::endl;

		return 0;
	} catch(const exception::CommonException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cerr << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
