/*
 * alangop.cpp
 *
 *  Created on: 20. 11. 2014
 *	  Author: Tomas Pecka
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <vector>
#include <sax/FromXMLParserHelper.h>

#include <factory/XmlDataFactory.hpp>
#include <exception/CommonException.h>
#include <automaton/Automaton.h>

#include <automaton/transform/AutomataConcatenationEpsilonTransition.h>
#include <automaton/transform/AutomataConcatenation.h>
#include <automaton/transform/AutomataIntersectionCartesianProduct.h>
#include <automaton/transform/AutomataUnionCartesianProduct.h>
#include <automaton/transform/AutomataUnionEpsilonTransition.h>
#include <automaton/transform/AutomatonIterationEpsilonTransition.h>
#include <automaton/transform/AutomatonIteration.h>


int main(int argc, char* argv[]) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd("Stringology algorithm access binary", ' ', "0.01");

		std::vector<std::string> allowed;
		allowed.push_back("unionEpsilon");
		allowed.push_back("unionCartesian");
		allowed.push_back("concatenationEpsilon");
		allowed.push_back("concatenation");
		allowed.push_back("intersectionCartesian");
		allowed.push_back("iterationEpsilon");
		allowed.push_back("iteration");
		TCLAP::ValuesConstraint<std::string> allowedVals( allowed );

		TCLAP::ValueArg<std::string> algorithm(	"a",	"algorithm",	"Execute algorithm", true,	"",	&allowedVals);
		cmd.add(algorithm);

		TCLAP::ValueArg<std::string> a1(	"i",	"input1",	"First automaton",	false,	"-",	"file");
		TCLAP::ValueArg<std::string> a2(	"j",	"input2",	"Second automaton",	false,	"-",	"file");
		cmd.add( a1 );
		cmd.add( a2 );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc,argv);

		if(verbose.isSet())
			common::GlobalData::verbose = true;
		if(measure.isSet())
			common::GlobalData::measure = true;

		measurements::start("Overal", measurements::Type::OVERALL);
		measurements::start("Input read", measurements::Type::AUXILIARY);

		automaton::Automaton automaton1 = alib::XmlDataFactory::fromTokens<automaton::Automaton>(sax::FromXMLParserHelper::parseInput(a1));

		if( algorithm.getValue() == "unionEpsilon") {
			automaton::Automaton automaton2 = alib::XmlDataFactory::fromTokens<automaton::Automaton>(sax::FromXMLParserHelper::parseInput(a1));

			measurements::end();
			measurements::start("Algorithm", measurements::Type::MAIN);

			automaton::Automaton res = automaton::transform::AutomataUnionEpsilonTransition::unification(automaton1, automaton2);

			measurements::end();
			measurements::start("Output write", measurements::Type::AUXILIARY);

			alib::XmlDataFactory::toStdout( res );

		} else if( algorithm.getValue() == "unionCartesian") {
			automaton::Automaton automaton2 = alib::XmlDataFactory::fromTokens<automaton::Automaton>(sax::FromXMLParserHelper::parseInput(a1));

			measurements::end();
			measurements::start("Algorithm", measurements::Type::MAIN);

			automaton::Automaton res = automaton::transform::AutomataUnionCartesianProduct::unification(automaton1, automaton2);

			measurements::end();
			measurements::start("Output write", measurements::Type::AUXILIARY);

			alib::XmlDataFactory::toStdout( res );
		} else if( algorithm.getValue() == "concatenationEpsilon") {
			automaton::Automaton automaton2 = alib::XmlDataFactory::fromTokens<automaton::Automaton>(sax::FromXMLParserHelper::parseInput(a1));

			measurements::end();
			measurements::start("Algorithm", measurements::Type::MAIN);

			automaton::Automaton res = automaton::transform::AutomataConcatenationEpsilonTransition::concatenation(automaton1, automaton2);

			measurements::end();
			measurements::start("Output write", measurements::Type::AUXILIARY);

			alib::XmlDataFactory::toStdout( res );
		} else if( algorithm.getValue() == "concatenation") {
			automaton::Automaton automaton2 = alib::XmlDataFactory::fromTokens<automaton::Automaton>(sax::FromXMLParserHelper::parseInput(a1));

			measurements::end();
			measurements::start("Algorithm", measurements::Type::MAIN);

			automaton::Automaton res = automaton::transform::AutomataConcatenation::concatenation(automaton1, automaton2);

			measurements::end();
			measurements::start("Output write", measurements::Type::AUXILIARY);

			alib::XmlDataFactory::toStdout( res );
		} else if( algorithm.getValue() == "intersectionCartesian") {
			automaton::Automaton automaton2 = alib::XmlDataFactory::fromTokens<automaton::Automaton>(sax::FromXMLParserHelper::parseInput(a1));

			measurements::end();
			measurements::start("Algorithm", measurements::Type::MAIN);

			automaton::Automaton res = automaton::transform::AutomataIntersectionCartesianProduct::intersection(automaton1, automaton2);

			measurements::end();
			measurements::start("Output write", measurements::Type::AUXILIARY);

			alib::XmlDataFactory::toStdout( res );
		} else if( algorithm.getValue() == "iteration") {

			measurements::end();
			measurements::start("Algorithm", measurements::Type::MAIN);

			automaton::Automaton res = automaton::transform::AutomatonIteration::iteration(automaton1);

			measurements::end();
			measurements::start("Output write", measurements::Type::AUXILIARY);

			alib::XmlDataFactory::toStdout( res );
		} else if( algorithm.getValue() == "iterationEpsilon") {

			measurements::end();
			measurements::start("Algorithm", measurements::Type::MAIN);

			automaton::Automaton res = automaton::transform::AutomatonIterationEpsilonTransition::iteration(automaton1);

			measurements::end();
			measurements::start("Output write", measurements::Type::AUXILIARY);

			alib::XmlDataFactory::toStdout( res );
		} else {
			throw exception::CommonException( "Invalid algorithm" );
		}

		measurements::end();
		measurements::end();

		if(measure.getValue()) std::cmeasure << measurements::results() << std::endl;

		return 0;
	} catch( const exception::CommonException & exception ) {
		alib::XmlDataFactory::toStdout( exception );
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cout << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
