/*
 * NFADeterminizer.cpp
 *
 *  Created on: 16. 1. 2014
 *	  Author: Jan Vesely
 */

#include "common/NFACommon.h"

#include <automaton/FSM/NFA.h>
#include <automaton/FSM/MultiInitialStateNFA.h>
#include <deque>
#include <algorithm>

namespace automaton {

namespace determinize {

automaton::DFA<> Determinize::determinize ( const automaton::MultiInitialStateNFA < > & nfa ) {
	 // 1, 4
	label::Label initialState ( createDFAState ( nfa.getInitialStates ( ) ) );
	automaton::DFA < > res ( initialState );

	res.setInputAlphabet ( nfa.getInputAlphabet ( ) );

	 // 2
	std::deque < label::Label > todo;
	todo.push_back ( std::move ( initialState ) );

	do {
		 // 3a, c
		label::Label state = std::move ( todo.front ( ) );
		todo.pop_front ( );

		 // 3b
		for ( const alphabet::Symbol & input : nfa.getInputAlphabet ( ) ) {
			std::set < label::Label > targetNFAStates;

			for ( label::Label nfaState : recreateNFAStates ( state ) ) {
				auto iter = nfa.getTransitions ( ).find ( std::make_pair ( std::move ( nfaState ), input ) );

				if ( iter != nfa.getTransitions ( ).end ( ) )
					targetNFAStates.insert ( iter->second.begin ( ), iter->second.end ( ) );
			}

			label::Label dfaState = createDFAState ( std::move ( targetNFAStates ) );

			 // 4
			bool existed = !res.addState ( dfaState );

			if ( !existed ) todo.push_back ( dfaState );

			 // 3b
			res.addTransition ( state, input, std::move ( dfaState ) );
		}
	} while ( !todo.empty ( ) );

	 // 5
	const std::set<label::Label>& finalLabels = nfa.getFinalStates();
	for ( const label::Label & dfaState : res.getStates ( ) ) {
		const std::set < label::Label > & nfaStates = recreateNFAStates ( dfaState );

		if(!std::excludes(finalLabels.begin(), finalLabels.end(), nfaStates.begin(), nfaStates.end()))
			res.addFinalState ( dfaState );
	}

	return res;
}

auto DeterminizeMultiInitialStateNFA = Determinize::RegistratorWrapper < automaton::DFA<>, automaton::MultiInitialStateNFA < > > ( Determinize::determinize );

automaton::DFA<> Determinize::determinize ( const automaton::NFA < > & nfa ) {
	 // 1, 4
	label::Label initialState ( createDFAState ( { nfa.getInitialState ( ) } ) );
	automaton::DFA < > res ( initialState );

	res.setInputAlphabet ( nfa.getInputAlphabet ( ) );

	 // 2
	std::deque < label::Label > todo;
	todo.push_back ( std::move ( initialState ) );

	do {
		 // 3a, c
		label::Label state = std::move ( todo.front ( ) );
		todo.pop_front ( );

		 // 3b
		for ( const alphabet::Symbol & input : nfa.getInputAlphabet ( ) ) {
			std::set < label::Label > targetNFAStates;

			for ( label::Label nfaState : recreateNFAStates ( state ) ) {
				auto iter = nfa.getTransitions ( ).find ( std::make_pair ( std::move ( nfaState ), input ) );

				if ( iter != nfa.getTransitions ( ).end ( ) )
					targetNFAStates.insert ( iter->second.begin ( ), iter->second.end ( ) );
			}

			label::Label dfaState = createDFAState ( std::move ( targetNFAStates ) );

			 // 4
			bool existed = !res.addState ( dfaState );

			if ( !existed ) todo.push_back ( dfaState );

			 // 3b
			res.addTransition ( state, input, std::move ( dfaState ) );
		}
	} while ( !todo.empty ( ) );

	 // 5
	const std::set<label::Label>& finalLabels = nfa.getFinalStates();
	for ( const label::Label & dfaState : res.getStates ( ) ) {
		const std::set < label::Label > & nfaStates = recreateNFAStates ( dfaState );

		if(!std::excludes(finalLabels.begin(), finalLabels.end(), nfaStates.begin(), nfaStates.end()))
			res.addFinalState ( dfaState );
	}

	return res;
}

auto DeterminizeNFA = Determinize::RegistratorWrapper < automaton::DFA<>, automaton::NFA < > > ( Determinize::determinize );

} /* namespace determinize */

} /* namespace automaton */
