/*
 * Occurrences.h
 *
 *  Created on: 9. 2. 2014
 *	  Author: Jan Travnicek
 */

#ifndef _AUTOMATON_OCCURRENCES_H__
#define _AUTOMATON_OCCURRENCES_H__

#include <core/multipleDispatch.hpp>
#include <automaton/Automaton.h>
#include <automaton/AutomatonFeatures.h>
#include <object/Object.h>
#include <string/LinearString.h>
#include <tree/ranked/RankedTree.h>

#include "Run.h"
#include <automaton/FSM/DFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/TA/DFTA.h>
#include <automaton/PDA/InputDrivenDPDA.h>
#include <automaton/PDA/VisiblyPushdownDPDA.h>
#include <automaton/PDA/RealTimeHeightDeterministicDPDA.h>
#include <automaton/PDA/DPDA.h>

#include <deque>

namespace automaton {

namespace run {

class Occurrences : public std::DoubleDispatch < Occurrences, std::set < unsigned >, const automaton::AutomatonBase &, const alib::ObjectBase & > {
public:
	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static std::set < unsigned > occurrences ( const automaton::Automaton & automaton, const alib::Object & object );
	static std::set < unsigned > occurrences ( const automaton::Automaton & automaton, const string::LinearString < > & string );
	static std::set < unsigned > occurrences ( const automaton::Automaton & automaton, const tree::RankedTree < > & string );

	template < class SymbolType, class StateType >
	static std::set < unsigned > occurrences ( const automaton::DFA < SymbolType, StateType > & automaton, const string::LinearString < SymbolType > & string );
	template < class SymbolType, class RankType, class StateType >
	static std::set < unsigned > occurrences ( const automaton::DFTA < SymbolType, RankType, StateType > & automaton, const tree::RankedTree < SymbolType, RankType > & tree );
	template < class SymbolType, class StateType >
	static std::set < unsigned > occurrences ( const automaton::InputDrivenDPDA < SymbolType, StateType > & automaton, const string::LinearString < SymbolType > & string );
	template < class SymbolType, class StateType >
	static std::set < unsigned > occurrences ( const automaton::VisiblyPushdownDPDA < SymbolType, StateType > & automaton, const string::LinearString < SymbolType > & string );
	template < class SymbolType, class EpsilonType, class StateType >
	static std::set < unsigned > occurrences ( const automaton::RealTimeHeightDeterministicDPDA < SymbolType, EpsilonType, StateType > & automaton, const string::LinearString < SymbolType > & string );
	template < class SymbolType, class EpsilonType, class StateType >
	static std::set < unsigned > occurrences ( const automaton::DPDA < SymbolType, EpsilonType, StateType > & automaton, const string::LinearString < SymbolType > & string );

};

template < class SymbolType, class StateType >
std::set < unsigned > Occurrences::occurrences ( const automaton::DFA < SymbolType, StateType > & automaton, const string::LinearString < SymbolType > & string ) {
	std::tuple < bool, StateType, std::set < unsigned > > res = Run::calculateState ( automaton, string );

	return std::get < 2 > ( res );
}

template < class SymbolType, class RankType, class StateType >
std::set < unsigned > Occurrences::occurrences ( const automaton::DFTA < SymbolType, RankType, StateType > & automaton, const tree::RankedTree < SymbolType, RankType > & tree ) {
	std::tuple < bool, StateType, std::set < unsigned > > res = Run::calculateState ( automaton, tree );

	return std::get < 2 > ( res );
}

template < class SymbolType, class StateType >
std::set < unsigned > Occurrences::occurrences ( const automaton::InputDrivenDPDA < SymbolType, StateType > & automaton, const string::LinearString < SymbolType > & string ) {
	std::tuple < bool, StateType, std::set < unsigned >, std::deque < SymbolType > > res = Run::calculateState ( automaton, string );

	return std::get < 2 > ( res );
}

template < class SymbolType, class StateType >
std::set < unsigned > Occurrences::occurrences ( const automaton::VisiblyPushdownDPDA < SymbolType, StateType > & automaton, const string::LinearString < SymbolType > & string ) {
	std::tuple < bool, StateType, std::set < unsigned >, std::deque < SymbolType > > res = Run::calculateState ( automaton, string );

	return std::get < 2 > ( res );
}

template < class SymbolType, class EpsilonType, class StateType >
std::set < unsigned > Occurrences::occurrences ( const automaton::RealTimeHeightDeterministicDPDA < SymbolType, EpsilonType, StateType > & automaton, const string::LinearString < SymbolType > & string ) {
	std::tuple < bool, StateType, std::set < unsigned >, std::deque < SymbolType > > res = Run::calculateState ( automaton, string );

	return std::get < 2 > ( res );
}

template < class SymbolType, class EpsilonType, class StateType >
std::set < unsigned > Occurrences::occurrences ( const automaton::DPDA < SymbolType, EpsilonType, StateType > & automaton, const string::LinearString < SymbolType > & string ) {
	std::tuple < bool, StateType, std::set < unsigned >, std::deque < SymbolType > > res = Run::calculateState ( automaton, string );

	return std::get < 2 > ( res );
}

} /* namespace run */

} /* namespace automaton */

#endif /* _AUTOMATON_OCCURRENCES_H__ */
