/*
 * ToAutomatonBototmUp.h
 *
 * Created on: 20. 5. 2015
 * Author: Tomas Pecka
 */

#ifndef _GRAMMAR_TO_AUTOMATON_BOTTOMUP_H_
#define _GRAMMAR_TO_AUTOMATON_BOTTOMUP_H_

#include <core/multipleDispatch.hpp>

#include <grammar/Grammar.h>
#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>

#include <automaton/Automaton.h>
#include <automaton/PDA/NPDA.h>

#include <alphabet/BottomOfTheStackSymbol.h>
#include <label/InitialStateLabel.h>
#include <label/FinalStateLabel.h>

namespace grammar {

namespace convert {

/**
 */
class ToAutomatonBottomUp : public std::SingleDispatch<ToAutomatonBottomUp, automaton::Automaton, const grammar::GrammarBase &> {
public:
	/**
	 * Performs conversion.
	 */
	static automaton::Automaton convert(const grammar::Grammar& grammar);

	template <class T, class SymbolType = typename grammar::SymbolTypeOfGrammar < T >, class EpsilonType = DefaultEpsilonType, class StateType = DefaultStateType >
	static automaton::NPDA < SymbolType, EpsilonType, StateType > convert(const T& grammar);
};

template <class T, class SymbolType, class EpsilonType, class StateType >
automaton::NPDA < SymbolType, EpsilonType, StateType > ToAutomatonBottomUp::convert(const T& grammar) {
	StateType q = label::InitialStateLabel::instance < StateType > ( );
	StateType r = label::FinalStateLabel::instance < StateType > ( );

	automaton::NPDA < SymbolType, EpsilonType, StateType > automaton(q, alphabet::BottomOfTheStackSymbol::instance < SymbolType > ( ) );
	automaton.addState(r);
	automaton.addFinalState(r);

	automaton.setInputAlphabet(grammar.getTerminalAlphabet());

	for(const SymbolType& symbol : grammar.getNonterminalAlphabet())
		automaton.addPushdownStoreSymbol(symbol);
	for(const SymbolType& symbol : grammar.getTerminalAlphabet())
		automaton.addPushdownStoreSymbol(symbol);

	for(const SymbolType& symbol : grammar.getTerminalAlphabet())
		automaton.addTransition(automaton.getInitialState(), symbol, std::vector<SymbolType>{}, automaton.getInitialState(), std::vector<SymbolType>{symbol});

	//for(const std::map<SymbolType, std::set<std::vector<SymbolType>>>& kv : grammar.getRules())
	for(const auto& kv : grammar.getRules())
		for(const std::vector<SymbolType>& rhs : kv.second)
			automaton.addTransition(automaton.getInitialState(), rhs, automaton.getInitialState(), std::vector<SymbolType>{kv.first});

	automaton.addTransition(automaton.getInitialState(), std::vector<SymbolType>{ alphabet::BottomOfTheStackSymbol::instance < SymbolType > ( ), grammar.getInitialSymbol()}, r, std::vector<SymbolType>{});

	return automaton;
}

} /* namespace convert */

} /* namespace grammar */

#endif /* _GRAMMAR_TO_AUTOMATON_BOTTOMUP_H_ */
