/*
 * ToAutomatonGlushkov.h
 *
 *  Created on: 11. 1. 2014
 *	  Author: Tomas Pecka
 */

#ifndef TO_AUTOMATON_GLUSHKOV_H_
#define TO_AUTOMATON_GLUSHKOV_H_

#include <core/multipleDispatch.hpp>
#include <map>

#include <automaton/FSM/NFA.h>
#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExp.h>
#include <regexp/unbounded/UnboundedRegExp.h>

namespace regexp {

namespace convert {

/**
 * Converts regular expression to finite automaton using Glushkov's NFA construction algorithm.
 * Source: Melichar 2.107
 */
class ToAutomatonGlushkov : public std::SingleDispatch<ToAutomatonGlushkov, automaton::NFA < >, const regexp::RegExpBase & > {
public:
	/**
	 * Performs conversion.
	 * @param re Original regular expression.
	 * @return NFA equivalent to original regular expression.
	 */
	static automaton::NFA < > convert(const regexp::RegExp& re);

	static automaton::NFA < > convert(const regexp::UnboundedRegExp < > & re);
	static automaton::NFA < > convert(const regexp::FormalRegExp < > & re);

};

} /* namespace convert */

} /* namespace regexp */

#endif /* TO_AUTOMATON_GLUSHKOV_H_ */
