/*
 * ToGrammarRightRGDerivation.h
 *
 *  Created on: 6. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef TO_GRAMMAR_RIGHT_RG_DERIVATION_H_
#define TO_GRAMMAR_RIGHT_RG_DERIVATION_H_

#include <core/multipleDispatch.hpp>

#include <grammar/Regular/RightRG.h>
#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExp.h>
#include <regexp/unbounded/UnboundedRegExp.h>

namespace regexp {

namespace convert {

/**
 * Converts reg. expression to right regular grammar using brzozowski derivation algorithm.
 * Source: Melichar 2.137
 */
class ToGrammarRightRGDerivation : public std::SingleDispatch<ToGrammarRightRGDerivation, grammar::RightRG < >, const regexp::RegExpBase &> {
public:
	/**
	 * Performs conversion.
	 * @return right regular grammar equivalent to source regexp.
	 */
	static grammar::RightRG < > convert(const regexp::RegExp& regexp);

	template <class T>
	static grammar::RightRG < > convert(const T& regexp);

};

} /* namespace convert */

} /* namespace regexp */

#endif /* TO_GRAMMAR_RIGHT_RG_DERIVATION_H_ */
