/*
 * ToGrammarRightRGGlushkov.cpp
 *
 *  Created on: 11. 1. 2014
 *	  Author: Tomas Pecka
 */

#include "ToGrammarRightRGGlushkov.h"

#include <algorithm>

#include <alphabet/InitialSymbol.h>

#include <label/LabelPairLabel.h>

#include "../glushkov/GlushkovFollow.h"
#include "../glushkov/GlushkovIndexate.h"
#include "../glushkov/GlushkovFirst.h"
#include "../glushkov/GlushkovLast.h"

#include "../properties/RegExpEpsilon.h"
#include <exception/CommonException.h>

namespace regexp {

namespace convert {

grammar::RightRG < > ToGrammarRightRGGlushkov::convert ( const regexp::RegExp & regexp ) {
	return dispatch ( regexp.getData ( ) );
}

grammar::RightRG < > ToGrammarRightRGGlushkov::convert ( const regexp::UnboundedRegExp < > & regexp ) {
	alphabet::Symbol S = alphabet::InitialSymbol::instance < alphabet::Symbol > ( );
	grammar::RightRG < > grammar ( S );

	 // step 1
	grammar.setTerminalAlphabet ( regexp.getAlphabet ( ) );

	regexp::UnboundedRegExp < > indexedRegExp = regexp::GlushkovIndexate::index ( regexp );

	 // steps 2, 3, 4
	const std::set < regexp::UnboundedRegExpSymbol < alphabet::Symbol > > first = regexp::GlushkovFirst::first ( indexedRegExp );
	const std::set < regexp::UnboundedRegExpSymbol < alphabet::Symbol > > last = regexp::GlushkovLast::last ( indexedRegExp );

	// \e in q0 check is in step 7

	 // step 5
	for ( const auto & symbol : indexedRegExp.getAlphabet ( ) )
		grammar.addNonterminalSymbol ( symbol );

	 // step 6
	for ( const auto & symbol : first )
		grammar.addRule ( S, std::make_pair ( regexp::GlushkovIndexate::getSymbolFromGlushkovPair ( symbol.getSymbol ( ) ), symbol.getSymbol ( ) ) );

	for ( const auto & x : indexedRegExp.getAlphabet ( ) )
		for ( const auto & f : regexp::GlushkovFollow::follow ( indexedRegExp, UnboundedRegExpSymbol < alphabet::Symbol > ( x ) ) ) {
			const alphabet::Symbol & a = x;
			const alphabet::Symbol & b = f.getSymbol ( );

			grammar.addRule ( a, std::make_pair ( regexp::GlushkovIndexate::getSymbolFromGlushkovPair ( b ), b ) );
		}

	// step 7

	/*
	 * for all rules where ns.m_nonTerminal is on rightSide:
	 *  add Rule: leftSide -> symbol.getSymbol( )
	 *  unless it already exists
	 */
	for ( const auto & rule : grammar.getRawRules ( ) ) {
		for ( const auto & rhs : rule.second )
			for ( const auto & symbol : last ) {

				const alphabet::Symbol & a = symbol.getSymbol ( );

				if ( std::find ( rhs.begin ( ), rhs.end ( ), a ) != rhs.end ( ) )
					grammar.addRule ( rule.first, rhs.at ( 0 ) );
			}

	}

	if ( regexp::properties::RegExpEpsilon::languageContainsEpsilon ( regexp ) )
		grammar.setGeneratesEpsilon ( true );

	return grammar;
}

auto ToGrammarRightRGGlushkovUnboundedRegExp = ToGrammarRightRGGlushkov::RegistratorWrapper < grammar::RightRG < >, regexp::UnboundedRegExp < > > ( ToGrammarRightRGGlushkov::convert );

grammar::RightRG < > ToGrammarRightRGGlushkov::convert ( const regexp::FormalRegExp < > & /* regexp */ ) {
	throw exception::CommonException ( "Glushkov: Converting FormalRegExp NYI" ); // TODO
}

auto ToGrammarRightRGGlushkovFormalRegExp = ToGrammarRightRGGlushkov::RegistratorWrapper < grammar::RightRG < >, regexp::FormalRegExp < > > ( ToGrammarRightRGGlushkov::convert );

} /* namespace convert */

} /* namespace regexp */
