/*
 * GlushkovIndexate.cpp
 *
 *  Created on: 13. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "GlushkovIndexate.h"

#include <regexp/unbounded/UnboundedRegExpAlternation.h>
#include <regexp/unbounded/UnboundedRegExpConcatenation.h>
#include <regexp/unbounded/UnboundedRegExpElement.h>
#include <regexp/unbounded/UnboundedRegExpEmpty.h>
#include <regexp/unbounded/UnboundedRegExpEpsilon.h>
#include <regexp/unbounded/UnboundedRegExpIteration.h>
#include <regexp/unbounded/UnboundedRegExpSymbol.h>

#include <alphabet/SymbolPairSymbol.h>

namespace regexp {

alphabet::Symbol GlushkovIndexate::getSymbolFromGlushkovPair ( const alphabet::Symbol & symbol ) {
	const alphabet::SymbolPairSymbol & symbolPair = ( const alphabet::SymbolPairSymbol & ) symbol.getData ( );

	return symbolPair.getData ( ).first;
}

UnboundedRegExp < > GlushkovIndexate::index ( const regexp::UnboundedRegExp < > & re ) {
	int i = 1;

	return UnboundedRegExp < > ( regexp::UnboundedRegExpStructure < alphabet::Symbol > ( re.getRegExp ( ).getStructure ( ).accept < std::rvalue_ref < regexp::UnboundedRegExpElement < alphabet::Symbol > >, GlushkovIndexate::Unbounded > ( i ) ) );
}

std::rvalue_ref < regexp::UnboundedRegExpElement < alphabet::Symbol > > GlushkovIndexate::Unbounded::visit(const regexp::UnboundedRegExpAlternation < alphabet::Symbol > & alternation, int & i) {
	UnboundedRegExpAlternation < alphabet::Symbol > * alt = new UnboundedRegExpAlternation < alphabet::Symbol > ( );

	for ( const auto & element : alternation.getElements ( ) )
		alt->appendElement ( element->accept < std::rvalue_ref < regexp::UnboundedRegExpElement < alphabet::Symbol > >, GlushkovIndexate::Unbounded > ( i ) );

	return std::rvalue_ref < UnboundedRegExpElement < alphabet::Symbol > > ( alt );
}

std::rvalue_ref < regexp::UnboundedRegExpElement < alphabet::Symbol > > GlushkovIndexate::Unbounded::visit(const regexp::UnboundedRegExpConcatenation < alphabet::Symbol > & concatenation, int & i) {
	UnboundedRegExpConcatenation < alphabet::Symbol > * con = new UnboundedRegExpConcatenation < alphabet::Symbol > ( );

	for ( const auto & element : concatenation.getElements ( ) )
		con->appendElement ( element->accept < std::rvalue_ref < regexp::UnboundedRegExpElement < alphabet::Symbol > >, GlushkovIndexate::Unbounded > ( i ) );

	return std::rvalue_ref < UnboundedRegExpElement < alphabet::Symbol > > ( con );
}

std::rvalue_ref < regexp::UnboundedRegExpElement < alphabet::Symbol > > GlushkovIndexate::Unbounded::visit(const regexp::UnboundedRegExpIteration < alphabet::Symbol > & iteration, int & i) {
	return std::rvalue_ref < UnboundedRegExpElement < alphabet::Symbol > > ( new UnboundedRegExpIteration < alphabet::Symbol > ( iteration.getElement ( ).accept < std::rvalue_ref < regexp::UnboundedRegExpElement < alphabet::Symbol > >, GlushkovIndexate::Unbounded > ( i ) ) );
}

std::rvalue_ref < regexp::UnboundedRegExpElement < alphabet::Symbol > > GlushkovIndexate::Unbounded::visit(const regexp::UnboundedRegExpSymbol < alphabet::Symbol > & symbol, int & i) {
	return std::rvalue_ref < UnboundedRegExpElement < alphabet::Symbol > > ( new UnboundedRegExpSymbol < alphabet::Symbol > ( alphabet::Symbol ( alphabet::SymbolPairSymbol ( std::make_pair ( symbol.getSymbol ( ), alphabet::Symbol ( i++ ) ) ) ) ) );
}

std::rvalue_ref < regexp::UnboundedRegExpElement < alphabet::Symbol > > GlushkovIndexate::Unbounded::visit(const regexp::UnboundedRegExpEpsilon < alphabet::Symbol > &, int &) {
	return std::rvalue_ref < UnboundedRegExpElement < alphabet::Symbol > > ( new UnboundedRegExpEpsilon < alphabet::Symbol > ( ) );
}

std::rvalue_ref < regexp::UnboundedRegExpElement < alphabet::Symbol > > GlushkovIndexate::Unbounded::visit(const regexp::UnboundedRegExpEmpty < alphabet::Symbol > &, int &) {
	return std::rvalue_ref < UnboundedRegExpElement < alphabet::Symbol > > ( new UnboundedRegExpEmpty < alphabet::Symbol > ( ) );
}

} /* namespace regexp */
