/*
 * GlushkovIndexate.h
 *
 *  Created on: 13. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef GLUSHKOV_INDEXATE_H_
#define GLUSHKOV_INDEXATE_H_

#include <regexp/unbounded/UnboundedRegExp.h>
#include <regexp/RegExpFeatures.h>

namespace regexp {

/**
 * RegExp tree traversal utils for Glushkov algorithm.
 *
 */
class GlushkovIndexate {
public:
	static alphabet::Symbol getSymbolFromGlushkovPair ( const alphabet::Symbol & symbol );

	/**
	 * @param re RegExp to index
	 * @return UnboundedRegExp with indexed elements
	 */
	static regexp::UnboundedRegExp < > index ( const regexp::UnboundedRegExp < > & re );

	class Unbounded {
	public:
		static std::rvalue_ref < regexp::UnboundedRegExpElement < alphabet::Symbol > > visit(const regexp::UnboundedRegExpAlternation < alphabet::Symbol > & alternation, int & i);
		static std::rvalue_ref < regexp::UnboundedRegExpElement < alphabet::Symbol > > visit(const regexp::UnboundedRegExpConcatenation < alphabet::Symbol > & concatenation, int & i);
		static std::rvalue_ref < regexp::UnboundedRegExpElement < alphabet::Symbol > > visit(const regexp::UnboundedRegExpIteration < alphabet::Symbol > & iteration, int & i);
		static std::rvalue_ref < regexp::UnboundedRegExpElement < alphabet::Symbol > > visit(const regexp::UnboundedRegExpSymbol < alphabet::Symbol > & symbol, int & i);
		static std::rvalue_ref < regexp::UnboundedRegExpElement < alphabet::Symbol > > visit(const regexp::UnboundedRegExpEpsilon < alphabet::Symbol > & epsilon, int & i);
		static std::rvalue_ref < regexp::UnboundedRegExpElement < alphabet::Symbol > > visit(const regexp::UnboundedRegExpEmpty < alphabet::Symbol > & empty, int & i);
	};
};

} /* namespace regexp */

#endif /* GLUSHKOV_INDEXATE_H_ */
