/*
 * GlushkovFirst.h
 *
 *  Created on: 14. 4. 2016
 *	  Author: Tomas Pecka
 */

#ifndef RTE_GLUSHKOV_FIRST_H_
#define RTE_GLUSHKOV_FIRST_H_

#include <set>

#include <rte/formal/FormalRTE.h>
#include <rte/RTEFeatures.h>

#include <alphabet/RankedSymbol.h>

namespace rte {

class GlushkovFirst {
public:
	/**
	 * @param re rte to probe
	 * @return all rteSymbols which can be root of the tree.
	 */
	template < class SymbolType, class RankType >
	static std::set < std::ranked_symbol < SymbolType, RankType > > first ( const rte::FormalRTE < SymbolType, RankType > & re );

	template < class SymbolType, class RankType >
	class Formal {
	public:
		static std::set < std::ranked_symbol < SymbolType, RankType > > visit ( const rte::FormalRTEAlternation < SymbolType, RankType > & node );
		static std::set < std::ranked_symbol < SymbolType, RankType > > visit ( const rte::FormalRTESubstitution < SymbolType, RankType > & node );
		static std::set < std::ranked_symbol < SymbolType, RankType > > visit ( const rte::FormalRTEIteration < SymbolType, RankType > & node );
		static std::set < std::ranked_symbol < SymbolType, RankType > > visit ( const rte::FormalRTESymbolAlphabet < SymbolType, RankType > & node );
		static std::set < std::ranked_symbol < SymbolType, RankType > > visit ( const rte::FormalRTESymbolSubst < SymbolType, RankType > & node );
		static std::set < std::ranked_symbol < SymbolType, RankType > > visit ( const rte::FormalRTEEmpty < SymbolType, RankType > & node );
	};
};

template < class SymbolType, class RankType >
std::set < std::ranked_symbol < SymbolType, RankType > > GlushkovFirst::first ( const rte::FormalRTE < SymbolType, RankType > & rte ) {
	return rte.getRTE ( ).getStructure ( ).template accept < std::set < std::ranked_symbol < SymbolType, RankType > >, GlushkovFirst::Formal < SymbolType, RankType > > ( );
}

template < class SymbolType, class RankType >
std::set < std::ranked_symbol < SymbolType, RankType > > GlushkovFirst::Formal < SymbolType, RankType >::visit ( const rte::FormalRTEAlternation < SymbolType, RankType > & node ) {
	std::set < std::ranked_symbol < SymbolType, RankType > > ret, tmp;

	tmp = node.getLeftElement ( ).template accept < std::set < std::ranked_symbol < SymbolType, RankType > >, GlushkovFirst::Formal < SymbolType, RankType > > ( );
	ret.insert ( tmp.begin ( ), tmp.end ( ) );

	tmp = node.getRightElement ( ).template accept < std::set < std::ranked_symbol < SymbolType, RankType > >, GlushkovFirst::Formal < SymbolType, RankType > > ( );
	ret.insert ( tmp.begin ( ), tmp.end ( ) );

	return ret;
}

template < class SymbolType, class RankType >
std::set < std::ranked_symbol < SymbolType, RankType > > GlushkovFirst::Formal < SymbolType, RankType >::visit ( const rte::FormalRTESubstitution < SymbolType, RankType > & node ) {
	std::set < std::ranked_symbol < SymbolType, RankType > > ret, tmp;

	tmp = node.getLeftElement ( ).template accept < std::set < std::ranked_symbol < SymbolType, RankType > >, GlushkovFirst::Formal < SymbolType, RankType > > ( );
	ret.insert ( tmp.begin ( ), tmp.end ( ) );

	 // First returns a set. hence only one occurrence.
	auto it = std::find_if ( ret.begin ( ), ret.end ( ), [node] ( const std::ranked_symbol < SymbolType, RankType > & a ) {
			return a == node.getSubstitutionSymbol ( ).getSymbol ( );
		} );

	if ( it != ret.end ( ) ) {
		ret.erase ( it );
		tmp = node.getRightElement ( ).template accept < std::set < std::ranked_symbol < SymbolType, RankType > >, GlushkovFirst::Formal < SymbolType, RankType > > ( );
		ret.insert ( tmp.begin ( ), tmp.end ( ) );
	}

	return ret;
}

template < class SymbolType, class RankType >
std::set < std::ranked_symbol < SymbolType, RankType > > GlushkovFirst::Formal < SymbolType, RankType >::visit ( const rte::FormalRTEIteration < SymbolType, RankType > & node ) {
	std::set < std::ranked_symbol < SymbolType, RankType > > ret;

	ret = node.getElement ( ).template accept < std::set < std::ranked_symbol < SymbolType, RankType > >, GlushkovFirst::Formal < SymbolType, RankType > > ( );
	ret.insert ( node.getSubstitutionSymbol ( ).getSymbol ( ) );
	return ret;
}

template < class SymbolType, class RankType >
std::set < std::ranked_symbol < SymbolType, RankType > > GlushkovFirst::Formal < SymbolType, RankType >::visit ( const rte::FormalRTESymbolAlphabet < SymbolType, RankType > & node ) {
	return std::set < std::ranked_symbol < SymbolType, RankType > > { node.getSymbol ( ) };
}

template < class SymbolType, class RankType >
std::set < std::ranked_symbol < SymbolType, RankType > > GlushkovFirst::Formal < SymbolType, RankType >::visit ( const rte::FormalRTESymbolSubst < SymbolType, RankType > & node ) {
	return std::set < std::ranked_symbol < SymbolType, RankType > > { node.getSymbol ( ) };
}

template < class SymbolType, class RankType >
std::set < std::ranked_symbol < SymbolType, RankType > > GlushkovFirst::Formal < SymbolType, RankType >::visit ( const rte::FormalRTEEmpty < SymbolType, RankType > & /* node */ ) {
	return std::set < std::ranked_symbol < SymbolType, RankType > > ( );
}

} /* namespace rte */

#endif /* RTE_GLUSHKOV_FIRST_H_ */
