#include "GrammarToCNFTest.h"

#include "grammar/simplify/ToCNF.h"

#include "grammar/ContextFree/CFG.h"
#include "grammar/ContextFree/CNF.h"

#include "alphabet/SymbolPairSymbol.h"
#include "alphabet/UniqueSymbol.h"
#include "primitive/Integer.h"

#include <pair>

#include <factory/StringDataFactory.hpp>

#define CPPUNIT_IMPLY(x, y) CPPUNIT_ASSERT(!(x) || (y))

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GrammarToCNFTest, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION( GrammarToCNFTest );

void GrammarToCNFTest::setUp() {
}

void GrammarToCNFTest::tearDown() {
}

void GrammarToCNFTest::testToCNFRules1() {
	alphabet::Symbol S = alphabet::Symbol("S");
	alphabet::Symbol A = alphabet::Symbol("A");
	alphabet::Symbol B = alphabet::Symbol("B");
	alphabet::Symbol C = alphabet::Symbol("C");
	alphabet::Symbol D = alphabet::Symbol("D");

	alphabet::Symbol a = alphabet::Symbol("a");
	alphabet::Symbol b = alphabet::Symbol("b");

	alphabet::Symbol aP = alphabet::Symbol("a'");
	alphabet::Symbol bP = alphabet::Symbol("b'");

	grammar::CFG < > grammar1(S);
	grammar1.setNonterminalAlphabet({S, A, B, C, D});
	grammar1.setTerminalAlphabet({a, b});

	grammar::CNF < > grammar2 = grammar::simplify::ToCNF::convert(grammar1);

	grammar::CNF < > grammar3(S);
	grammar3.setNonterminalAlphabet({S, A, B, C, D, aP, bP});
	grammar3.setTerminalAlphabet({a, b});
	grammar3.addRule(aP, a);
	grammar3.addRule(bP, b);

	std::cout << grammar2 << std::endl;
	std::cout << grammar3 << std::endl;

	CPPUNIT_ASSERT(grammar2 == grammar3);
}

void GrammarToCNFTest::testToCNFRules2() {
	alphabet::Symbol S = alphabet::Symbol("S");
	alphabet::Symbol X = alphabet::Symbol("X");
	alphabet::Symbol Y = alphabet::Symbol("Y");

	alphabet::Symbol a = alphabet::Symbol("a");
	alphabet::Symbol b = alphabet::Symbol("b");
	alphabet::Symbol c = alphabet::Symbol("c");

	grammar::CFG < > grammar1(S);
	grammar1.setNonterminalAlphabet({S, X, Y});
	grammar1.setTerminalAlphabet({a, b, c});
	grammar1.addRule({S}, {a, X, b, X});
	grammar1.addRule({X}, {a, Y});
	grammar1.addRule({X}, {b, Y});
	grammar1.addRule({X}, {});
	grammar1.addRule({Y}, {X});
	grammar1.addRule({Y}, {c});

	grammar::CNF < > grammar2 = grammar::simplify::ToCNF::convert(grammar1);
	alphabet::Symbol aP = alphabet::Symbol("a'");
	alphabet::Symbol bP = alphabet::Symbol("b'");
	alphabet::Symbol cP = alphabet::Symbol("c'");
	alphabet::Symbol Xb = alphabet::Symbol(alphabet::Symbol(alphabet::SymbolPairSymbol(std::make_pair(X, b))));
	alphabet::Symbol aX = alphabet::Symbol(alphabet::Symbol(alphabet::SymbolPairSymbol(std::make_pair(a, X))));
	alphabet::Symbol bX = alphabet::Symbol(alphabet::Symbol(alphabet::SymbolPairSymbol(std::make_pair(b, X))));

	grammar::CNF < > grammar3(S);
	grammar3.setNonterminalAlphabet({S, X, Y, aP, bP, cP, Xb, aX, bX});
	grammar3.setTerminalAlphabet({a, b, c});
	grammar3.addRule(S, std::make_pair(aX, bX));
	grammar3.addRule(S, std::make_pair(aP, bX));
	grammar3.addRule(S, std::make_pair(aP, Xb));
	grammar3.addRule(S, std::make_pair(aP, bP));
	grammar3.addRule(aX, std::make_pair(aP, X));
	grammar3.addRule(bX, std::make_pair(bP, X));
	grammar3.addRule(Xb, std::make_pair(X, bP));
	grammar3.addRule(X, std::make_pair(aP, Y));
	grammar3.addRule(X, std::make_pair(bP, Y));
	grammar3.addRule(X, a);
	grammar3.addRule(X, b);
	grammar3.addRule(Y, std::make_pair(aP, Y));
	grammar3.addRule(Y, std::make_pair(bP, Y));
	grammar3.addRule(Y, a);
	grammar3.addRule(Y, b);
	grammar3.addRule(Y, c);
	grammar3.addRule(aP, a);
	grammar3.addRule(bP, b);
	grammar3.addRule(cP, c);

	std::cout << alib::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar2)) << std::endl;
	std::cout << alib::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar3)) << std::endl;

	CPPUNIT_ASSERT(grammar2 == grammar3);
}

