#include <list>
#include "rg2rgTest.h"

#include "grammar/convert/ToGrammarLeftRG.h"
#include "grammar/convert/ToGrammarRightRG.h"

#define CPPUNIT_IMPLY(x, y) CPPUNIT_ASSERT(!(x) || (y))

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( rg2rgTest, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION( rg2rgTest );

void rg2rgTest::setUp() {
}

void rg2rgTest::tearDown() {
}

void rg2rgTest::testConversion() {
  grammar::RightRG < > rrGrammar(alphabet::Symbol(1));

  rrGrammar.addNonterminalSymbol(alphabet::Symbol(1));
  rrGrammar.addNonterminalSymbol(alphabet::Symbol(2));
  rrGrammar.addNonterminalSymbol(alphabet::Symbol(3));
  rrGrammar.addTerminalSymbol(alphabet::Symbol("a"));
  rrGrammar.addTerminalSymbol(alphabet::Symbol("b"));
  
  rrGrammar.addRule(alphabet::Symbol(1), std::make_pair(alphabet::Symbol("a"), alphabet::Symbol(2)));
  rrGrammar.addRule(alphabet::Symbol(2), std::make_pair(alphabet::Symbol("b"), alphabet::Symbol(3)));
  rrGrammar.addRule(alphabet::Symbol(3), alphabet::Symbol("a"));

  grammar::LeftRG < > lrGrammar = grammar::convert::ToGrammarLeftRG::convert(rrGrammar);

  grammar::LeftRG < > lrGrammarRef(alphabet::Symbol(4));

  lrGrammarRef.addNonterminalSymbol(alphabet::Symbol(1));
  lrGrammarRef.addNonterminalSymbol(alphabet::Symbol(2));
  lrGrammarRef.addNonterminalSymbol(alphabet::Symbol(3));
  lrGrammarRef.addTerminalSymbol(alphabet::Symbol("a"));
  lrGrammarRef.addTerminalSymbol(alphabet::Symbol("b"));

  lrGrammarRef.addRule(alphabet::Symbol(2), std::make_pair(alphabet::Symbol(1), alphabet::Symbol("a")));
  lrGrammarRef.addRule(alphabet::Symbol(2), alphabet::Symbol("a"));
  lrGrammarRef.addRule(alphabet::Symbol(3), std::make_pair(alphabet::Symbol(2), alphabet::Symbol("b")));
  lrGrammarRef.addRule(alphabet::Symbol(4), std::make_pair(alphabet::Symbol(3), alphabet::Symbol("a")));

  CPPUNIT_ASSERT(lrGrammarRef == lrGrammar);
}

void rg2rgTest::testConversion2() {
  grammar::LeftRG < > lrGrammar(alphabet::Symbol(4));

  lrGrammar.addNonterminalSymbol(alphabet::Symbol(2));
  lrGrammar.addNonterminalSymbol(alphabet::Symbol(3));
  lrGrammar.addNonterminalSymbol(alphabet::Symbol(4));
  lrGrammar.addTerminalSymbol(alphabet::Symbol("a"));
  lrGrammar.addTerminalSymbol(alphabet::Symbol("b"));

  lrGrammar.addRule(alphabet::Symbol(2), alphabet::Symbol("a"));
  lrGrammar.addRule(alphabet::Symbol(3), std::make_pair(alphabet::Symbol(2), alphabet::Symbol("b")));
  lrGrammar.addRule(alphabet::Symbol(4), std::make_pair(alphabet::Symbol(3), alphabet::Symbol("a")));

  grammar::RightRG < > rrGrammar = grammar::convert::ToGrammarRightRG::convert(lrGrammar);

  grammar::RightRG < > rrGrammarRef(alphabet::Symbol(5));

  rrGrammarRef.addNonterminalSymbol(alphabet::Symbol(2));
  rrGrammarRef.addNonterminalSymbol(alphabet::Symbol(3));
  rrGrammarRef.addNonterminalSymbol(alphabet::Symbol(4));
  rrGrammarRef.addTerminalSymbol(alphabet::Symbol("a"));
  rrGrammarRef.addTerminalSymbol(alphabet::Symbol("b"));

  rrGrammarRef.addRule(alphabet::Symbol(5), std::make_pair(alphabet::Symbol("a"), alphabet::Symbol(2)));
  rrGrammarRef.addRule(alphabet::Symbol(2), std::make_pair(alphabet::Symbol("b"), alphabet::Symbol(3)));
  rrGrammarRef.addRule(alphabet::Symbol(3), alphabet::Symbol("a"));
  rrGrammarRef.addRule(alphabet::Symbol(3), std::make_pair(alphabet::Symbol("a"), alphabet::Symbol(4)));

  CPPUNIT_ASSERT(rrGrammarRef == rrGrammar);
}
