#include "AbsorbTerminalSymbol.h"

#include "grammar/ContextFree/CFG.h"
#include <alphabet/SymbolPairSymbol.h>
#include "grammar/parsing/AbsorbTerminalSymbol.h"
#include "factory/StringDataFactory.hpp"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( AbsorbTerminalSymbol, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION ( AbsorbTerminalSymbol );

void AbsorbTerminalSymbol::setUp ( ) {
}

void AbsorbTerminalSymbol::tearDown ( ) {
}

void AbsorbTerminalSymbol::testAbsorbTerminalSymbol ( ) {
	alphabet::Symbol A = alphabet::Symbol ( "A" );
	alphabet::Symbol B = alphabet::Symbol ( "B" );
	alphabet::Symbol C = alphabet::Symbol ( "C" );

	alphabet::Symbol a = alphabet::Symbol ( 'a' );
	alphabet::Symbol b = alphabet::Symbol ( 'b' );
	alphabet::Symbol c = alphabet::Symbol ( 'c' );

	alphabet::Symbol Ba = alphabet::Symbol ( alphabet::SymbolPairSymbol ( std::make_pair ( B, a ) ) );

	grammar::CFG < > grammar ( A );

	grammar.setTerminalAlphabet ( { a, b, c } );
	grammar.setNonterminalAlphabet ( { A, B, C } );
	grammar.setInitialSymbol ( A );

	grammar.addRule ( A, { B, a, C } );
	grammar.addRule ( B, { } );
	grammar.addRule ( B, { a, a, C } );
	grammar.addRule ( C, { c } );
	grammar.addRule ( C, { b, C } );

	grammar::CFG < > res = grammar;
	grammar::parsing::AbsorbTerminalSymbol::absorbTerminalSymbol ( res, a, { B } );

	grammar::CFG < > comp ( A );

	comp.setTerminalAlphabet ( { a, b, c } );
	comp.setNonterminalAlphabet ( { A, B, Ba, C } );
	comp.setInitialSymbol ( A );

	comp.addRule ( A, { Ba, C } );
	comp.addRule ( B, { } );
	comp.addRule ( B, { a, a, C } );
	comp.addRule ( Ba, { a } );
	comp.addRule ( Ba, { a, a, C, a } );
	comp.addRule ( C, { c } );
	comp.addRule ( C, { b, C } );

	// std::cout << alib::StringDataFactory::toString ( grammar::Grammar ( res ) ) << std::endl << alib::StringDataFactory::toString ( grammar::Grammar ( comp ) ) << std::endl;

	CPPUNIT_ASSERT ( res == comp );
}

void AbsorbTerminalSymbol::testAbsorbTerminalSymbol2 ( ) {
	alphabet::Symbol A = alphabet::Symbol ( "A" );
	alphabet::Symbol B = alphabet::Symbol ( "B" );
	alphabet::Symbol C = alphabet::Symbol ( "C" );
	alphabet::Symbol X = alphabet::Symbol ( "X" );

	alphabet::Symbol a = alphabet::Symbol ( 'a' );
	alphabet::Symbol b = alphabet::Symbol ( 'b' );
	alphabet::Symbol c = alphabet::Symbol ( 'c' );

	alphabet::Symbol Ba = alphabet::Symbol ( alphabet::SymbolPairSymbol ( std::make_pair ( B, a ) ) );
	alphabet::Symbol Xa = alphabet::Symbol ( alphabet::SymbolPairSymbol ( std::make_pair ( X, a ) ) );

	grammar::CFG < > grammar ( A );

	grammar.setTerminalAlphabet ( { a, b, c } );
	grammar.setNonterminalAlphabet ( { A, B, C, X } );
	grammar.setInitialSymbol ( A );

	grammar.addRule ( A, { X, a, C } );
	grammar.addRule ( X, { c, B } );
	grammar.addRule ( B, { } );
	grammar.addRule ( B, { a, a, C } );
	grammar.addRule ( C, { c } );
	grammar.addRule ( C, { b, C } );

	grammar::CFG < > res = grammar;
	grammar::parsing::AbsorbTerminalSymbol::absorbTerminalSymbol ( res, a, { B, X } );

	grammar::CFG < > comp ( A );

	comp.setTerminalAlphabet ( { a, b, c } );
	comp.setNonterminalAlphabet ( { A, B, Ba, C, X, Xa } );
	comp.setInitialSymbol ( A );

	comp.addRule ( A, { Xa, C } );
	comp.addRule ( X, { c, B } );
	comp.addRule ( Xa, { c, Ba } );
	comp.addRule ( B, { } );
	comp.addRule ( B, { a, a, C } );
	comp.addRule ( Ba, { a } );
	comp.addRule ( Ba, { a, a, C, a } );
	comp.addRule ( C, { c } );
	comp.addRule ( C, { b, C } );

	// std::cout << "res " << alib::StringDataFactory::toString ( grammar::Grammar ( res ) ) << std::endl << "comp " << alib::StringDataFactory::toString ( grammar::Grammar ( comp ) ) << std::endl;

	CPPUNIT_ASSERT ( res == comp );
}

void AbsorbTerminalSymbol::testAbsorbTerminalSymbol3 ( ) {
	alphabet::Symbol A = alphabet::Symbol ( "A" );
	alphabet::Symbol B = alphabet::Symbol ( "B" );
	alphabet::Symbol C = alphabet::Symbol ( "C" );

	alphabet::Symbol a = alphabet::Symbol ( 'a' );
	alphabet::Symbol b = alphabet::Symbol ( 'b' );
	alphabet::Symbol c = alphabet::Symbol ( 'c' );

	alphabet::Symbol Ba = alphabet::Symbol ( alphabet::SymbolPairSymbol ( std::make_pair ( B, a ) ) );

	grammar::CFG < > grammar ( A );

	grammar.setTerminalAlphabet ( { a, b, c } );
	grammar.setNonterminalAlphabet ( { A, B, C } );
	grammar.setInitialSymbol ( A );

	grammar.addRule ( A, { B, a, C } );
	grammar.addRule ( B, { } );
	grammar.addRule ( B, { a, a, B } );
	grammar.addRule ( C, { c } );
	grammar.addRule ( C, { b, C } );

	grammar::CFG < > res = grammar;
	grammar::parsing::AbsorbTerminalSymbol::absorbTerminalSymbol ( res, a, { B } );

	grammar::CFG < > comp ( A );

	comp.setTerminalAlphabet ( { a, b, c } );
	comp.setNonterminalAlphabet ( { A, B, Ba, C } );
	comp.setInitialSymbol ( A );

	comp.addRule ( A, { Ba, C } );
	comp.addRule ( B, { } );
	comp.addRule ( B, { a, a, B } );
	comp.addRule ( Ba, { a } );
	comp.addRule ( Ba, { a, a, Ba } );
	comp.addRule ( C, { c } );
	comp.addRule ( C, { b, C } );

	// std::cout << alib::StringDataFactory::toString ( grammar::Grammar ( res ) ) << std::endl << alib::StringDataFactory::toString ( grammar::Grammar ( comp ) ) << std::endl;

	CPPUNIT_ASSERT ( res == comp );
}
