#include "FirstTest.h"

#include "grammar/parsing/First.h"
#include "string/Epsilon.h"
#include "grammar/ContextFree/CFG.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( FirstTest, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION ( FirstTest );

void FirstTest::setUp ( ) {
}

void FirstTest::tearDown ( ) {
}

void FirstTest::testFirst ( ) {
	{
		DefaultSymbolType nE = DefaultSymbolType ( 'E' );
		DefaultSymbolType nT = DefaultSymbolType ( 'T' );
		DefaultSymbolType nF = DefaultSymbolType ( 'F' );

		DefaultSymbolType tP = DefaultSymbolType ( '+' );
		DefaultSymbolType tS = DefaultSymbolType ( '*' );
		DefaultSymbolType tL = DefaultSymbolType ( '(' );
		DefaultSymbolType tR = DefaultSymbolType ( ')' );
		DefaultSymbolType tA = DefaultSymbolType ( 'a' );

		grammar::CFG < > grammar ( nE );
		grammar.setTerminalAlphabet ( std::set < DefaultSymbolType > { tP, tS, tL, tR, tA } );
		grammar.setNonterminalAlphabet ( std::set < DefaultSymbolType > { nE, nT, nF } );

		std::vector < DefaultSymbolType > rhsE1 ( { nE, tP, nT } );
		std::vector < DefaultSymbolType > rhsE2 ( { nT } );
		std::vector < DefaultSymbolType > rhsT1 ( { nT, tS, nF } );
		std::vector < DefaultSymbolType > rhsT2 ( { nF } );
		std::vector < DefaultSymbolType > rhsF1 ( { tA } );
		std::vector < DefaultSymbolType > rhsF2 ( { tL, nE, tR } );

		grammar.addRule ( nE, rhsE1 );
		grammar.addRule ( nE, rhsE2 );
		grammar.addRule ( nT, rhsT1 );
		grammar.addRule ( nT, rhsT2 );
		grammar.addRule ( nF, rhsF1 );
		grammar.addRule ( nF, rhsF2 );

		 // --------------------------------------------------
		std::map < std::vector < DefaultSymbolType >, std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > > first;

		first[rhsE1] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tA, tL
		};
		first[rhsE2] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tA, tL
		};
		first[rhsT1] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tA, tL
		};
		first[rhsT2] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tA, tL
		};
		first[rhsF1] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tA
		};
		first[rhsF2] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tL
		};

		// --------------------------------------------------

		std::map < std::vector < DefaultSymbolType >, std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > > firstAlgo;

		for ( const auto & rule : grammar.getRawRules ( ) )
			for ( const auto & rhs : rule.second )
				firstAlgo[rhs] = grammar::parsing::First::first ( grammar, rhs );

		CPPUNIT_ASSERT ( first == firstAlgo );
	}

	{
		DefaultSymbolType nS = DefaultSymbolType ( 'S' );
		DefaultSymbolType nA = DefaultSymbolType ( 'A' );
		DefaultSymbolType nB = DefaultSymbolType ( 'B' );
		DefaultSymbolType nC = DefaultSymbolType ( 'C' );
		DefaultSymbolType nD = DefaultSymbolType ( 'D' );
		DefaultSymbolType nE = DefaultSymbolType ( 'E' );
		DefaultSymbolType nF = DefaultSymbolType ( 'F' );

		DefaultSymbolType tA = DefaultSymbolType ( 'a' );
		DefaultSymbolType tB = DefaultSymbolType ( 'b' );
		DefaultSymbolType tC = DefaultSymbolType ( 'c' );
		DefaultSymbolType tD = DefaultSymbolType ( 'd' );
		DefaultSymbolType tE = DefaultSymbolType ( 'e' );

		grammar::CFG < > grammar ( nS );
		grammar.setTerminalAlphabet ( std::set < DefaultSymbolType > { tA, tB, tC, tD, tE } );
		grammar.setNonterminalAlphabet ( std::set < DefaultSymbolType > { nS, nA, nB, nC, nD, nE, nF } );

		std::vector < DefaultSymbolType > rhsS1 ( { nB, tD, nS } );
		std::vector < DefaultSymbolType > rhsS2 ( { tD, tD, nC } );
		std::vector < DefaultSymbolType > rhsS3 ( { tC, nA } );
		std::vector < DefaultSymbolType > rhsA1 ( { tA, tE, nE } );
		std::vector < DefaultSymbolType > rhsA2 ( { tB, tB, nE } );
		std::vector < DefaultSymbolType > rhsB1 ( { tA, nF } );
		std::vector < DefaultSymbolType > rhsB2 ( { tB, tB, nD } );
		std::vector < DefaultSymbolType > rhsC1 ( { tA, nB, tD } );
		std::vector < DefaultSymbolType > rhsC2 ( { tE, nA } );
		std::vector < DefaultSymbolType > rhsD1 ( { tC, tA, nF } );
		std::vector < DefaultSymbolType > rhsE1 ( { tC, tA, tE, nE } );
		std::vector < DefaultSymbolType > rhsE2 ( { } );
		std::vector < DefaultSymbolType > rhsF1 ( { tE, nD } );
		std::vector < DefaultSymbolType > rhsF2 ( { } );

		grammar.addRule ( nS, rhsS1 );
		grammar.addRule ( nS, rhsS2 );
		grammar.addRule ( nS, rhsS3 );
		grammar.addRule ( nA, rhsA1 );
		grammar.addRule ( nA, rhsA2 );
		grammar.addRule ( nB, rhsB1 );
		grammar.addRule ( nB, rhsB2 );
		grammar.addRule ( nC, rhsC1 );
		grammar.addRule ( nC, rhsC2 );
		grammar.addRule ( nD, rhsD1 );
		grammar.addRule ( nE, rhsE1 );
		grammar.addRule ( nE, rhsE2 );
		grammar.addRule ( nF, rhsF1 );
		grammar.addRule ( nF, rhsF2 );

		 // --------------------------------------------------
		std::map < std::vector < DefaultSymbolType >, std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > > first;

		first[rhsS1] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tA, tB
		};
		first[rhsS2] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tD
		};
		first[rhsS3] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tC
		};
		first[rhsA1] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tA
		};
		first[rhsA2] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tB
		};
		first[rhsB1] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tA
		};
		first[rhsB2] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tB
		};
		first[rhsC1] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tA
		};
		first[rhsC2] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tE
		};
		first[rhsD1] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tC
		};
		first[rhsE1] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tC
		};
		first[rhsE2] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			string::Epsilon < >::EPSILON
		};
		first[rhsF1] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			tE
		};
		first[rhsF2] = std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > {
			string::Epsilon < >::EPSILON
		};

		// --------------------------------------------------

		std::map < std::vector < DefaultSymbolType >, std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > > firstAlgo;

		for ( const auto & rule : grammar.getRawRules ( ) )
			for ( const auto & rhs : rule.second )
				firstAlgo[rhs] = grammar::parsing::First::first ( grammar, rhs );

		CPPUNIT_ASSERT ( first == firstAlgo );
	}
}

void FirstTest::testFirst2 ( ) {
	DefaultSymbolType A = DefaultSymbolType ( 'A' );
	DefaultSymbolType c = DefaultSymbolType ( 'c' );
	DefaultSymbolType d = DefaultSymbolType ( 'd' );

	grammar::CFG < > grammar ( A );

	grammar.setTerminalAlphabet ( { c, d } );
	grammar.setNonterminalAlphabet ( { { A } } );
	grammar.setInitialSymbol ( A );

	grammar.addRule ( A, std::vector < DefaultSymbolType > { A, c } );
	grammar.addRule ( A, std::vector < DefaultSymbolType > { d } );

	std::map < std::vector < DefaultSymbolType >, std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > > res = { { { d }, { d } }, { { A, c }, { d } } };
	CPPUNIT_ASSERT ( res == grammar::parsing::First::first ( grammar ) );
}

void FirstTest::testFirst3 ( ) {
	DefaultSymbolType S = DefaultSymbolType ( 'S' );
	DefaultSymbolType A = DefaultSymbolType ( 'A' );
	DefaultSymbolType B = DefaultSymbolType ( 'B' );
	DefaultSymbolType a = DefaultSymbolType ( 'a' );
	DefaultSymbolType b = DefaultSymbolType ( 'b' );
	DefaultSymbolType c = DefaultSymbolType ( 'c' );
	DefaultSymbolType d = DefaultSymbolType ( 'd' );
	DefaultSymbolType f = DefaultSymbolType ( 'f' );

	grammar::CFG < > grammar ( S );

	grammar.setTerminalAlphabet ( { a, b, c, d, f } );
	grammar.setNonterminalAlphabet ( { { S, A, B } } );
	grammar.setInitialSymbol ( S );

	grammar.addRule ( S, std::vector < DefaultSymbolType > { A, a } );
	grammar.addRule ( S, std::vector < DefaultSymbolType > { b, S } );
	grammar.addRule ( A, std::vector < DefaultSymbolType > { c, A, d } );
	grammar.addRule ( A, std::vector < DefaultSymbolType > { B } );
	grammar.addRule ( B, std::vector < DefaultSymbolType > { f, S } );
	grammar.addRule ( B, std::vector < DefaultSymbolType > { } );

	std::map < std::vector < DefaultSymbolType >, std::set < std::variant < DefaultSymbolType, string::Epsilon < > > > > res =
	{
		{ { A, a }, { c, f, a } }, { { b, S }, { b } }, { { c, A, d }, { c } }, { { B }, { f, string::Epsilon < >::EPSILON } }, { { f, S }, { f } }, { { }, { string::Epsilon < >::EPSILON } }
	};
	CPPUNIT_ASSERT ( res == grammar::parsing::First::first ( grammar ) );
}
