#include "LeftFactorize.h"

#include "grammar/ContextFree/CFG.h"
#include "grammar/parsing/LeftFactorize.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( LeftFactorize, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION ( LeftFactorize );

void LeftFactorize::setUp ( ) {
}

void LeftFactorize::tearDown ( ) {
}

void LeftFactorize::testLeftFactorize ( ) {
	DefaultSymbolType S = DefaultSymbolType ( "S" );
	DefaultSymbolType Sp = DefaultSymbolType ( "S'" );
	DefaultSymbolType B = DefaultSymbolType ( "B" );
	DefaultSymbolType Bp = DefaultSymbolType ( "B'" );
	DefaultSymbolType C = DefaultSymbolType ( "C" );
	DefaultSymbolType Cp = DefaultSymbolType ( "C'" );

	DefaultSymbolType a = DefaultSymbolType ( 'a' );
	DefaultSymbolType b = DefaultSymbolType ( 'b' );
	DefaultSymbolType c = DefaultSymbolType ( 'c' );

	grammar::CFG < > grammar ( S );

	grammar.setTerminalAlphabet ( { a, b, c } );
	grammar.setNonterminalAlphabet ( { S, B, C } );
	grammar.setInitialSymbol ( S );

	grammar.addRule ( S, std::vector < DefaultSymbolType > { a, B } );
	grammar.addRule ( S, std::vector < DefaultSymbolType > { a, C } );
	grammar.addRule ( B, std::vector < DefaultSymbolType > { b, B } );
	grammar.addRule ( B, std::vector < DefaultSymbolType > { b } );
	grammar.addRule ( C, std::vector < DefaultSymbolType > { c, C } );
	grammar.addRule ( C, std::vector < DefaultSymbolType > { c } );

	grammar::CFG < > res = grammar;
	grammar::parsing::LeftFactorize::leftFactorize ( res, a, S);
	grammar::parsing::LeftFactorize::leftFactorize ( res, b, B);
	grammar::parsing::LeftFactorize::leftFactorize ( res, c, C);

	grammar::CFG < > comp ( S );

	comp.setTerminalAlphabet ( { a, b, c } );
	comp.setNonterminalAlphabet ( { S, Sp, B, Bp, C, Cp } );
	comp.setInitialSymbol ( S );

	comp.addRule ( S, std::vector < DefaultSymbolType > { a, Sp } );
	comp.addRule ( Sp, std::vector < DefaultSymbolType > { B } );
	comp.addRule ( Sp, std::vector < DefaultSymbolType > { C } );
	comp.addRule ( B, std::vector < DefaultSymbolType > { b, Bp } );
	comp.addRule ( Bp, std::vector < DefaultSymbolType > { B } );
	comp.addRule ( Bp, std::vector < DefaultSymbolType > { } );
	comp.addRule ( C, std::vector < DefaultSymbolType > { c, Cp } );
	comp.addRule ( Cp, std::vector < DefaultSymbolType > { C } );
	comp.addRule ( Cp, std::vector < DefaultSymbolType > { } );

//	std::cout << res << std::endl << comp << std::endl;

	CPPUNIT_ASSERT ( res == comp );
}

void LeftFactorize::testLeftFactorize2 ( ) {
	DefaultSymbolType S = DefaultSymbolType ( "S" );
	DefaultSymbolType Sp = DefaultSymbolType ( "S'" );
	DefaultSymbolType B = DefaultSymbolType ( "B" );
	DefaultSymbolType Bp = DefaultSymbolType ( "B'" );
	DefaultSymbolType C = DefaultSymbolType ( "C" );
	DefaultSymbolType Cp = DefaultSymbolType ( "C'" );
	DefaultSymbolType D = DefaultSymbolType ( "D" );

	DefaultSymbolType a = DefaultSymbolType ( 'a' );
	DefaultSymbolType b = DefaultSymbolType ( 'b' );
	DefaultSymbolType c = DefaultSymbolType ( 'c' );
	DefaultSymbolType d = DefaultSymbolType ( 'd' );

	grammar::CFG < > grammar ( S );

	grammar.setTerminalAlphabet ( { a, b, c, d } );
	grammar.setNonterminalAlphabet ( { S, B, C, D } );
	grammar.setInitialSymbol ( S );

	grammar.addRule ( S, std::vector < DefaultSymbolType > { a, B } );
	grammar.addRule ( S, std::vector < DefaultSymbolType > { a, C } );
	grammar.addRule ( S, std::vector < DefaultSymbolType > { D } );
	grammar.addRule ( B, std::vector < DefaultSymbolType > { b, B } );
	grammar.addRule ( B, std::vector < DefaultSymbolType > { b } );
	grammar.addRule ( C, std::vector < DefaultSymbolType > { c, C } );
	grammar.addRule ( C, std::vector < DefaultSymbolType > { c } );
	grammar.addRule ( C, std::vector < DefaultSymbolType > { b } );
	grammar.addRule ( D, std::vector < DefaultSymbolType > { d } );

	grammar::CFG < > res = grammar;
	grammar::parsing::LeftFactorize::leftFactorize ( res, a, S);
	grammar::parsing::LeftFactorize::leftFactorize ( res, b, B);
	grammar::parsing::LeftFactorize::leftFactorize ( res, c, C);

	grammar::CFG < > comp ( S );

	comp.setTerminalAlphabet ( { a, b, c, d } );
	comp.setNonterminalAlphabet ( { S, Sp, B, Bp, C, Cp, D} );
	comp.setInitialSymbol ( S );

	comp.addRule ( S, std::vector < DefaultSymbolType > { a, Sp } );
	comp.addRule ( S, std::vector < DefaultSymbolType > { D } );
	comp.addRule ( Sp, std::vector < DefaultSymbolType > { B } );
	comp.addRule ( Sp, std::vector < DefaultSymbolType > { C } );
	comp.addRule ( B, std::vector < DefaultSymbolType > { b, Bp } );
	comp.addRule ( Bp, std::vector < DefaultSymbolType > { B } );
	comp.addRule ( Bp, std::vector < DefaultSymbolType > { } );
	comp.addRule ( C, std::vector < DefaultSymbolType > { c, Cp } );
	comp.addRule ( C, std::vector < DefaultSymbolType > { b } );
	comp.addRule ( Cp, std::vector < DefaultSymbolType > { C } );
	comp.addRule ( Cp, std::vector < DefaultSymbolType > { } );
	comp.addRule ( D, std::vector < DefaultSymbolType > { d } );

//	std::cout << res << std::endl << comp << std::endl;

	CPPUNIT_ASSERT ( res == comp );
}

