#include "FactorOracleAutomatonTest.h"

#include "string/LinearString.h"
#include "stringology/exact/FactorOracleAutomaton.h"
#include "stringology/exact/BackwardOracleMatching.h"
#include "stringology/exact/ExactFactorMatch.h"

#include "string/generate/RandomStringFactory.h"
#include "string/generate/RandomSubstringFactory.h"

#define CPPUNIT_IMPLY( x, y )    CPPUNIT_ASSERT ( !( x ) || ( y ) )

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( FactorOracleAutomatonTest, "stringology" );
CPPUNIT_TEST_SUITE_REGISTRATION ( FactorOracleAutomatonTest );

void FactorOracleAutomatonTest::setUp ( ) {
}

void FactorOracleAutomatonTest::tearDown ( ) {
}

void FactorOracleAutomatonTest::testFactorOracleConstruction ( ) {

    string::LinearString < > pattern ( "atatac" );

    automaton::DFA<> oracle = stringology::exact::FactorOracleAutomaton::construct ( pattern );

    automaton::DFA<> refOracle ( label::Label ( 0 ) );

    refOracle.addFinalState ( label::Label ( 0 ) );

    refOracle.setInputAlphabet ( pattern.getAlphabet ( ) );

    for ( int i = 1; i <= 6; ++i ) {
        refOracle.addState ( label::Label ( i ) );
        refOracle.addFinalState ( label::Label ( i ) );
    }

    refOracle.addTransition ( label::Label ( 0 ), alphabet::Symbol ( 'a' ), label::Label ( 1 ) );
    refOracle.addTransition ( label::Label ( 0 ), alphabet::Symbol ( 't' ), label::Label ( 2 ) );
    refOracle.addTransition ( label::Label ( 0 ), alphabet::Symbol ( 'c' ), label::Label ( 6 ) );
    refOracle.addTransition ( label::Label ( 1 ), alphabet::Symbol ( 't' ), label::Label ( 2 ) );
    refOracle.addTransition ( label::Label ( 1 ), alphabet::Symbol ( 'c' ), label::Label ( 6 ) );
    refOracle.addTransition ( label::Label ( 2 ), alphabet::Symbol ( 'a' ), label::Label ( 3 ) );
    refOracle.addTransition ( label::Label ( 3 ), alphabet::Symbol ( 't' ), label::Label ( 4 ) );
    refOracle.addTransition ( label::Label ( 3 ), alphabet::Symbol ( 'c' ), label::Label ( 6 ) );
    refOracle.addTransition ( label::Label ( 4 ), alphabet::Symbol ( 'a' ), label::Label ( 5 ) );
    refOracle.addTransition ( label::Label ( 5 ), alphabet::Symbol ( 'c' ), label::Label ( 6 ) );

    CPPUNIT_ASSERT ( oracle == refOracle );
}

void FactorOracleAutomatonTest::testBackwardOracleMatching ( ) {
    std::vector<std::string> subjects;
    std::vector<std::string> patterns;
    std::vector<std::set<unsigned>> expectedOccs;

    subjects.push_back("a"); patterns.push_back("a"); expectedOccs.push_back({0});
    subjects.push_back("a"); patterns.push_back("b"); expectedOccs.push_back({});
    subjects.push_back("alfalfalfa"); patterns.push_back("alfalfalfa"); expectedOccs.push_back({0});
    subjects.push_back("alfalfalfa"); patterns.push_back("blfalfalfa"); expectedOccs.push_back({});
    subjects.push_back("alfalfalfa"); patterns.push_back("alfalfalfb"); expectedOccs.push_back({});
    subjects.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfa"); patterns.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfa"); expectedOccs.push_back({0});
    subjects.push_back("alfalfalfaalfalfalfaabfalfalfaalfalfalfaalfalfalfaalfalfalfaalfa"); patterns.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfa"); expectedOccs.push_back({});
    subjects.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfaa"); patterns.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfaa"); expectedOccs.push_back({0});
    subjects.push_back("atggccttgcc"); patterns.push_back("gcc"); expectedOccs.push_back({3,8});
    subjects.push_back("aaaaaaaaaa"); patterns.push_back("a"); expectedOccs.push_back({0,1,2,3,4,5,6,7,8,9});


    for(size_t i = 0; i < subjects.size(); ++i) {
        string::String subject = string::stringFrom ( subjects[i] );
        string::String pattern = string::stringFrom ( patterns[i] );
        std::set < unsigned > res = stringology::exact::BackwardOracleMatching::match ( subject, pattern );
        std::cout << subjects[i] << ' ' << patterns[i] << ' ' << res << std::endl;
        CPPUNIT_ASSERT ( res == expectedOccs[i] );
    }

    auto longSubject = string::generate::RandomStringFactory::generateLinearString (64 * 64, 512, false, true);
    auto longPattern = string::generate::RandomSubstringFactory::generateSubstring(64 * 32, longSubject );
    std::set < unsigned > res = stringology::exact::BackwardOracleMatching::match ( longSubject, longPattern );
    std::set < unsigned > resRef = stringology::exact::ExactFactorMatch::match ( longSubject, longPattern );
    std::cout << "long: " << res << std::endl;
    CPPUNIT_ASSERT ( res == resRef);
}
