/*
 * createUnique.hpp
 *
 *  Created on: Nov 11, 2016
 *      Author: Jan Travnicek
 */

#ifndef CREATE_UNIQUE_H_
#define CREATE_UNIQUE_H_

#include <type_traits>
#include <set>
#include "exception/CommonException.h"
#include <climits>

namespace common {

// TODO repace in c++11 with fold expansion
template < class ... Ts >
inline bool andAll ( Ts ... );

template < >
inline bool andAll ( ) {
	return true;
}

template < class T, class ... Ts >
inline bool andAll ( T value, Ts ... other ) {
	return value && andAll ( other ... );
}

/**
 * Creates and adds unique state to grammar. If given state name is
 * already used, appends apostrophe or integer suffix
 * @param name name of the state
 * @throws AutomatonException if symbol could not be created
 * @return created symbol
 */
template < class T, class ... Alphabets >
typename std::enable_if < std::all_same < std::set < T >, Alphabets ... >::value, T >::type createUnique ( const T & base, const Alphabets & ... alphabets ) {
	T attempt = base;
	unsigned i = 0;

	do {
		// TODO repace in c++11 with fold expansion
		if ( andAll ( alphabets.count ( attempt ) == 0 ... ) )
			return attempt;

		++attempt;
	} while ( i++ < INT_MAX );

	throw exception::CommonException ( "Could not create unique symbol." );
}

}

#endif /* CREATE_UNIQUE_H_ */
