/*
 * FromXMLParserHelper.cpp
 *
 *  Created on: Oct 12, 2013
 *      Author: Jan Travnicek
 */

#include "FromXMLParserHelper.h"
#include "ParserException.h"
#include "../exception/CommonException.h"
#include "SaxParseInterface.h"

namespace sax {

bool FromXMLParserHelper::isToken(const std::deque<Token>::const_iterator& input, Token::TokenType type, const std::string& data) {
	return input->getType() == type && input->getData() == data;
}

bool FromXMLParserHelper::isTokenType(const std::deque<Token>::const_iterator& input, Token::TokenType type) {
	return input->getType() == type;
}

void FromXMLParserHelper::popToken(std::deque<Token>::iterator& input, Token::TokenType type, const std::string& data) {
	if (isToken(input, type, data)) {
		++input;
	} else {
		throw ParserException(Token(data, type), *input);
	}
}

std::string FromXMLParserHelper::popTokenData(std::deque<Token>::iterator& input, Token::TokenType type) {
	if(isTokenType(input, type)) {
		std::string result = std::move(*input).moveData();
		++input;
		return result;
	} else {
		throw ParserException(Token("?", type), *input);
	}
}

std::string FromXMLParserHelper::getTokenData(const std::deque<Token>::const_iterator& input, Token::TokenType type) {
	if(isTokenType(input, type)) {
		return input->getData();
	} else {
		throw ParserException(Token("?", type), *input);
	}
}

std::deque< sax::Token > FromXMLParserHelper::parseSingle(const std::string& input) {
	std::deque < sax::Token > res;
	sax::SaxParseInterface::parseFile ( input, res );
	return res;
}

std::deque < std::deque < sax::Token > > FromXMLParserHelper::parseInput(bool needSingle, /*const*/ TCLAP::MultiArg < std::string >& input) {
	std::deque < std::deque < sax::Token > > inputTokens;

	if ( input.isSet ( ) ) {
		if(needSingle && input.getValue().size() > 1)
			throw exception::CommonException("Multiple parameters when single required.");

		for ( const std::string & fileName : input.getValue() ) {
			inputTokens.emplace_back ( parseSingle ( fileName ) );
		}
	} else {
		inputTokens.emplace_back ( parseSingle ( "-" ) );
	}

	return inputTokens;
}

std::deque < sax::Token > FromXMLParserHelper::parseInput(/*const*/ TCLAP::ValueArg < std::string >& input) {
	if ( input.isSet ( ) ) {
		return parseSingle ( input.getValue() );
	} else {
		return parseSingle ( "-" );
	}
}
} /* namespace sax */
