/*
 * LabeledSymbol.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Martin Zak
 */

#include "LabeledSymbol.h"
#include <sax/FromXMLParserHelper.h>
#include "Symbol.h"
#include <object/Object.h>
#include <core/xmlApi.hpp>

namespace alphabet {

LabeledSymbol::LabeledSymbol(int number) : m_label(label::Label(number)) {

}

LabeledSymbol::LabeledSymbol(char character) : m_label(label::Label(character)) {

}

LabeledSymbol::LabeledSymbol(std::string label) : m_label(label::Label(std::move(label))) {

}

LabeledSymbol::LabeledSymbol(label::Label label) : m_label(std::move(label)) {

}

SymbolBase* LabeledSymbol::clone() const {
	return new LabeledSymbol(*this);
}

SymbolBase* LabeledSymbol::plunder() && {
	return new LabeledSymbol(std::move(*this));
}

const label::Label& LabeledSymbol::getLabel() const {
	return m_label;
}

label::Label& LabeledSymbol::getLabel() {
	return m_label;
}

int LabeledSymbol::compare(const LabeledSymbol& other) const {
	return this->m_label.getData().compare(other.m_label.getData());
}

void LabeledSymbol::operator>>(std::ostream& out) const {
	out << "(LabeledSymbol " << this->m_label << ")";
}

LabeledSymbol::operator std::string () const {
	return (std::string) m_label;
}

LabeledSymbol LabeledSymbol::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, LabeledSymbol::getXmlTagName());
	label::Label data = alib::xmlApi<label::Label>::parse(input);
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, LabeledSymbol::getXmlTagName());
	return LabeledSymbol(std::move(data));
}

void LabeledSymbol::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(LabeledSymbol::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<label::Label>::compose(out, m_label);
	out.emplace_back(LabeledSymbol::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

SymbolBase* LabeledSymbol::inc() && {
	m_label.inc();
	return NULL;
}

} /* namespace alphabet */

namespace alib {

auto labeledSymbolParserRegister = xmlApi<alphabet::Symbol>::ParserRegister<alphabet::LabeledSymbol>();
auto labeledSymbolParserRegister2 = xmlApi<alib::Object>::ParserRegister<alphabet::LabeledSymbol>();

} /* namespace alib */
