/*
 * NonlinearVariableSymbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef NONLINEAR_VARIABLE_SYMBOL_H_
#define NONLINEAR_VARIABLE_SYMBOL_H_

#include <sax/FromXMLParserHelper.h>

#include "Symbol.h"
#include "SymbolFeatures.h"
#include "UniqueSymbol.h"

namespace alphabet {

/**
 * Represents subtreeWildcard symbol for tree linearization.
 */
template < class SymbolType >
class NonlinearVariableSymbol : public SymbolBase {
	SymbolType m_symbol;

public:
	explicit NonlinearVariableSymbol ( int number );
	explicit NonlinearVariableSymbol ( char character );
	explicit NonlinearVariableSymbol ( std::string symbol );
	explicit NonlinearVariableSymbol ( SymbolType symbol );

	/**
	 * Creates a subtreeWildcard symbol.
	 * @param symbol name of the symbol
	 */
	explicit NonlinearVariableSymbol ( );

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	const SymbolType & getSymbol ( ) const;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const NonlinearVariableSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "NonlinearVariableSymbol";

		return xmlTagName;
	}

	static NonlinearVariableSymbol < SymbolType > parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual SymbolBase * inc ( ) &&;
};

template < class SymbolType >
NonlinearVariableSymbol < SymbolType >::NonlinearVariableSymbol ( int number ) : m_symbol ( alphabet::Symbol ( number ) ) {
}

template < class SymbolType >
NonlinearVariableSymbol < SymbolType >::NonlinearVariableSymbol ( char character ) : m_symbol ( alphabet::Symbol ( character ) ) {
}

template < class SymbolType >
NonlinearVariableSymbol < SymbolType >::NonlinearVariableSymbol ( std::string symbol ) : m_symbol ( alphabet::Symbol ( std::move ( symbol ) ) ) {
}

template < class SymbolType >
NonlinearVariableSymbol < SymbolType >::NonlinearVariableSymbol ( SymbolType symbol ) : m_symbol ( std::move ( symbol ) ) {
}

template < class SymbolType >
SymbolBase * NonlinearVariableSymbol < SymbolType >::clone ( ) const {
	return new NonlinearVariableSymbol ( * this );
}

template < class SymbolType >
SymbolBase * NonlinearVariableSymbol < SymbolType >::plunder ( ) && {
	return new NonlinearVariableSymbol ( std::move ( * this ) );
}

template < class SymbolType >
const SymbolType& NonlinearVariableSymbol < SymbolType >::getSymbol() const {
	return m_symbol;
}

template < class SymbolType >
int NonlinearVariableSymbol < SymbolType >::compare ( const NonlinearVariableSymbol & other ) const {
	std::compare < decltype ( m_symbol ) > comp;

	return comp ( m_symbol, other.m_symbol );
}

template < class SymbolType >
void NonlinearVariableSymbol < SymbolType >::operator >>( std::ostream & out ) const {
	out << "(NonlinearVariableSymbol " << m_symbol << ")";
}

template < class SymbolType >
NonlinearVariableSymbol < SymbolType >::operator std::string ( ) const {
	return "$" + std::to_string ( m_symbol );
}

template < class SymbolType >
NonlinearVariableSymbol < SymbolType > NonlinearVariableSymbol < SymbolType >::parse ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, NonlinearVariableSymbol::getXmlTagName() );
	SymbolType data = alib::xmlApi < SymbolType >::parse ( input );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, NonlinearVariableSymbol::getXmlTagName() );

	return NonlinearVariableSymbol < SymbolType > ( data );
}

template < class SymbolType >
void NonlinearVariableSymbol < SymbolType >::compose ( std::deque < sax::Token > & out ) const {
	out.emplace_back ( NonlinearVariableSymbol::getXmlTagName(), sax::Token::TokenType::START_ELEMENT );
	alib::xmlApi < SymbolType >::compose ( out, m_symbol );
	out.emplace_back ( NonlinearVariableSymbol::getXmlTagName(), sax::Token::TokenType::END_ELEMENT );
}

template < class SymbolType >
SymbolBase * NonlinearVariableSymbol < SymbolType >::inc ( ) && {
	return new UniqueSymbol ( Symbol ( std::move ( * this ) ), primitive::Integer ( 0 ) );
}

} /* namespace alphabet */

#endif /* NONLINEAR_VARIABLE_SYMBOL_H_ */
