/*
 * SymbolSetSymbol.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "SymbolSetSymbol.h"
#include <sstream>
#include <sax/FromXMLParserHelper.h>
#include <object/Object.h>
#include <core/xmlApi.hpp>
#include "UniqueSymbol.h"

namespace alphabet {

SymbolSetSymbol::SymbolSetSymbol(std::set<Symbol> raw) : symbol(std::move(raw)) {

}

SymbolBase* SymbolSetSymbol::clone() const {
	return new SymbolSetSymbol(*this);
}

SymbolBase* SymbolSetSymbol::plunder() && {
	return new SymbolSetSymbol(std::move(*this));
}

const std::set<Symbol>& SymbolSetSymbol::getData() const {
	return symbol;
}

int SymbolSetSymbol::compare(const SymbolSetSymbol& other) const {
	std::compare<std::set<Symbol>> comp;
	return comp(symbol, other.symbol);
}

void SymbolSetSymbol::operator>>(std::ostream& out) const {
	out << "(SymbolSetSymbol " << symbol << ")";
}

SymbolSetSymbol::operator std::string() const {
	std::stringstream ss;
	ss << "{";
	bool first = true;
	for(const Symbol& subSymbol : symbol) {
		if(!first) 
			ss << ", ";
		else
			first = false;
		ss << (std::string) subSymbol;
	}
	ss << "}";
	return std::move(ss).str();
}

SymbolSetSymbol SymbolSetSymbol::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, SymbolSetSymbol::getXmlTagName());
	std::set<Symbol> symbols;
	while(sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		symbols.insert(alib::xmlApi<Symbol>::parse(input));
	}
	SymbolSetSymbol data(std::move(symbols));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, SymbolSetSymbol::getXmlTagName());
	return data;
}

void SymbolSetSymbol::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(SymbolSetSymbol::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	for(const Symbol& innerSymbol : symbol) {
		alib::xmlApi<alphabet::Symbol>::compose(out, innerSymbol);
	}
	out.emplace_back(SymbolSetSymbol::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

SymbolBase* SymbolSetSymbol::inc() && {
	return new UniqueSymbol(Symbol(std::move(*this)), primitive::Integer(0));
}

} /* namespace alphabet */

namespace alib {

auto symbolSetSymbolParserRegister = xmlApi<alphabet::Symbol>::ParserRegister<alphabet::SymbolSetSymbol>();
auto symbolSetSymbolParserRegister2 = xmlApi<alib::Object>::ParserRegister<alphabet::SymbolSetSymbol>();

} /* namespace alib */
