/*
 * Label.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Martin Zak
 */

#include "Label.h"
#include "PrimitiveLabel.h"
#include "LabelPairLabel.h"
#include <primitive/Primitive.h>
#include <core/xmlApi.hpp>
#include <object/Object.h>
#include "ObjectLabel.h"
#include <alphabet/Symbol.h>

namespace label {

void Label::inc ( ) {
	this->operator ++ ();
}

Label::Label ( const alphabet::Symbol & symbol ) : alib::WrapperBase < LabelBase > ( label::ObjectLabel ( alib::Object ( symbol.getData ( ) ) ) ) {
}

Label::Label ( alphabet::Symbol && symbol ) : alib::WrapperBase < LabelBase > ( label::ObjectLabel ( alib::Object ( std::move ( symbol.getData ( ) ) ) ) ) {
}

Label::Label ( int number ) : alib::WrapperBase < LabelBase > ( label::PrimitiveLabel ( primitive::Primitive ( number ) ) ) {
}

Label::Label ( int number1, int number2 ) : Label ( label::Label { label::PrimitiveLabel { primitive::Primitive ( number1 ) } }, label::Label { label::PrimitiveLabel { primitive::Primitive ( number2 ) } } ) {
}

Label::Label ( label::Label label1, label::Label label2 ) : alib::WrapperBase < LabelBase > ( label::LabelPairLabel { std::make_pair ( std::move ( label1 ), std::move ( label2 ) ) } ) {
}

Label::Label ( char character ) : alib::WrapperBase < LabelBase > ( label::PrimitiveLabel { primitive::Primitive ( character ) } ) {
}

Label::Label ( std::string string ) : alib::WrapperBase < LabelBase > ( label::PrimitiveLabel { primitive::Primitive ( std::move ( string ) ) } ) {
}

Label::Label ( const char * string ) : Label ( ( std::string ) string ) {
}

} /* namespace label */

namespace alib {

auto LabelDeleter = xmlApi < label::Label >::InputContextDeleter ( );

}
