/*
 * LabelPairLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "LabelPairLabel.h"
#include <sstream>
#include <sax/FromXMLParserHelper.h>
#include "Label.h"
#include <object/Object.h>
#include <core/xmlApi.hpp>
#include "UniqueLabel.h"

namespace label {

LabelPairLabel::LabelPairLabel(std::pair<Label, Label> raw) : label(std::move(raw)) {

}

LabelBase* LabelPairLabel::clone() const {
	return new LabelPairLabel(*this);
}

LabelBase* LabelPairLabel::plunder() && {
	return new LabelPairLabel(std::move(*this));
}

const std::pair<Label, Label>& LabelPairLabel::getData() const {
	return label;
}

int LabelPairLabel::compare(const LabelPairLabel& other) const {
	int res = label.first.getData().compare(other.label.first.getData());
	if(res == 0) res = label.second.getData().compare(other.label.second.getData());
	return res;
}

void LabelPairLabel::operator>>(std::ostream& out) const {
	out << "(LabelPairLabel " << label << ")";
}

LabelPairLabel::operator std::string() const {
	std::stringstream ss;
	ss << "<";
	ss << (std::string) label.first;
	ss << ",";
	ss << (std::string) label.second;
	ss << "}";
	return std::move(ss).str();
}

LabelPairLabel LabelPairLabel::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, LabelPairLabel::getXmlTagName());

	Label firstLabel = alib::xmlApi<Label>::parse(input);
	Label secondLabel = alib::xmlApi<Label>::parse(input);

	LabelPairLabel data(std::make_pair(std::move(firstLabel), std::move(secondLabel)));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, LabelPairLabel::getXmlTagName());
	return data;
}

void LabelPairLabel::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(LabelPairLabel::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<Label>::compose(out, label.first);
	alib::xmlApi<Label>::compose(out, label.second);
	out.emplace_back(LabelPairLabel::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

LabelBase* LabelPairLabel::inc() && {
	return new UniqueLabel(Label(std::move(*this)), primitive::Integer(0));
}

} /* namespace label */

namespace alib {

auto labelPairLabelParserRegister = xmlApi<label::Label>::ParserRegister<label::LabelPairLabel>();
auto labelPairLabelParserRegister2 = xmlApi<alib::Object>::ParserRegister<label::LabelPairLabel>();

} /* namespace alib */
