/*
 * LabelSetLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "LabelSetLabel.h"
#include <sstream>
#include <sax/FromXMLParserHelper.h>
#include "Label.h"
#include <object/Object.h>
#include <core/xmlApi.hpp>
#include "UniqueLabel.h"

namespace label {

LabelSetLabel::LabelSetLabel(std::set<Label> raw) : label(std::move(raw)) {

}

LabelBase* LabelSetLabel::clone() const {
	return new LabelSetLabel(*this);
}

LabelBase* LabelSetLabel::plunder() && {
	return new LabelSetLabel(std::move(*this));
}

const std::set<Label>& LabelSetLabel::getData() const {
	return label;
}

int LabelSetLabel::compare(const LabelSetLabel& other) const {
	std::compare<std::set<Label>> comp;
	return comp(label, other.label);
}

void LabelSetLabel::operator>>(std::ostream& out) const {
	out << "(LabelSetLabel " << label << ")";
}

LabelSetLabel::operator std::string() const {
	std::stringstream ss;
	ss << "{";
	bool first = true;
	for(const Label& subLabel : label) {
		if(!first) 
			ss << ", ";
		else
			first = false;
		ss << (std::string) subLabel;
	}
	ss << "}";
	return std::move(ss).str();
}

LabelSetLabel LabelSetLabel::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, LabelSetLabel::getXmlTagName());
	std::set<Label> labels;
	while(sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		labels.insert(alib::xmlApi<Label>::parse(input));
	}
	LabelSetLabel data(std::move(labels));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, LabelSetLabel::getXmlTagName());
	return data;
}

void LabelSetLabel::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(LabelSetLabel::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	for(const Label& innerLabel : label) {
		alib::xmlApi<label::Label>::compose(out, innerLabel);
	}
	out.emplace_back(LabelSetLabel::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

LabelBase* LabelSetLabel::inc() && {
	return new UniqueLabel(Label(std::move(*this)), primitive::Integer(0));
}

} /* namespace label */

namespace alib {

auto labelSetLabelParserRegister = xmlApi<label::Label>::ParserRegister<label::LabelSetLabel>();
auto labelSetLabelParserRegister2 = xmlApi<alib::Object>::ParserRegister<label::LabelSetLabel>();

} /* namespace alib */
