/*
 * RegExp.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Jan Travnicek
 */

#include "RegExp.h"
#include <common/DefaultSymbolType.h>
#include "../string/LinearString.h"
#include "unbounded/UnboundedRegExpSymbol.h"
#include "unbounded/UnboundedRegExpEmpty.h"
#include "unbounded/UnboundedRegExpConcatenation.h"

#include "unbounded/UnboundedRegExp.h"
#include "unbounded/UnboundedRegExpStructure.h"
#include <core/xmlApi.hpp>

namespace regexp {

regexp::RegExp regexpFrom ( const std::string & string ) {
	return regexpFrom ( string::LinearString < > ( string ) );
}

regexp::RegExp regexpFrom ( const char * string ) {
	return regexpFrom ( string::LinearString < > ( string ) );
}

regexp::RegExp regexpFrom ( std::vector < DefaultSymbolType > string ) {
	regexp::UnboundedRegExpConcatenation < DefaultSymbolType > con;

	for ( auto & symbol : string )
		con.appendElement ( regexp::UnboundedRegExpSymbol < DefaultSymbolType > ( symbol ) );

	return regexp::RegExp { regexp::UnboundedRegExp < > ( regexp::UnboundedRegExpStructure < DefaultSymbolType > ( std::move ( con ) ) ) };
}

regexp::RegExp regexpFrom ( string::LinearString < > string ) {
	regexp::UnboundedRegExpConcatenation < DefaultSymbolType > con;

	for ( auto & symbol : string.getContent ( ) )
		con.appendElement ( regexp::UnboundedRegExpSymbol < DefaultSymbolType > ( symbol ) );

	return regexp::RegExp { regexp::UnboundedRegExp < > ( regexp::UnboundedRegExpStructure < DefaultSymbolType > ( std::move ( con ) ) ) };
}

regexp::RegExp regexpFrom ( DefaultSymbolType symbol ) {
	return regexp::RegExp { regexp::UnboundedRegExp < > ( regexp::UnboundedRegExpStructure < DefaultSymbolType > ( regexp::UnboundedRegExpSymbol < DefaultSymbolType > ( std::move ( symbol ) ) ) ) };
}

regexp::RegExp regexpFrom ( ) {
	return regexp::RegExp { regexp::UnboundedRegExp < > ( regexp::UnboundedRegExpStructure < DefaultSymbolType > ( regexp::UnboundedRegExpEmpty < DefaultSymbolType > { } ) ) };
}

} /* namespace regexp */

namespace alib {

auto RegExpDeleter = xmlApi < regexp::RegExp >::InputContextDeleter ( );

}
