/*
 * FormalRTEAlternation.h
 *
 *  Created on: Feb 20, 2016
 *      Author: Tomas Pecka
 */

#ifndef FORMAL_RTE_ALTERNATION_H_
#define FORMAL_RTE_ALTERNATION_H_

#include <sstream>

#include "FormalRTEElement.h"

#include <exception/CommonException.h>

namespace rte {

/**
 * Represents alternation operator in the regular expression. Contains list of FormalRTEElement
 * as operands of the operator.
 */
template < class SymbolType, class RankType >
class FormalRTEAlternation : public FormalRTEElement < SymbolType, RankType >, public std::BinaryNode < std::smart_ptr < FormalRTEElement < SymbolType, RankType > >, std::smart_ptr < FormalRTEElement < SymbolType, RankType > >, FormalRTEAlternation < SymbolType, RankType > > {
public:
	void accept ( typename FormalRTEElement < SymbolType, RankType >::Visitor & visitor ) const {
		visitor.visit ( * this );
	}

	explicit FormalRTEAlternation ( FormalRTEElement < SymbolType, RankType > && left, FormalRTEElement < SymbolType, RankType > && right );
	explicit FormalRTEAlternation ( const FormalRTEElement < SymbolType, RankType > & left, const FormalRTEElement < SymbolType, RankType > & right );

	/**
	 * @copydoc FormalRTEElement::clone() const
	 */
	virtual FormalRTEAlternation < SymbolType, RankType > * clone ( ) const;

	/**
	 * @copydoc FormalRTEElement::plunder() const
	 */
	virtual FormalRTEAlternation < SymbolType, RankType > * plunder ( ) &&;

	/**
	 * @copydoc FormalRTEElement::testSymbol() const
	 */
	virtual bool testSymbol ( const std::ranked_symbol < SymbolType, RankType > & symbol ) const;

	/**
	 * @copydoc FormalRTEElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetF, std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetK ) const;

	/**
	 * @copydoc FormalRTEElement::checkAlphabet()
	 */
	virtual bool checkAlphabet ( const std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetF, const std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetK ) const;

	/**
	 * @return elements
	 */
	const FormalRTEElement < SymbolType, RankType > & getLeftElement ( ) const;
	const FormalRTEElement < SymbolType, RankType > & getRightElement ( ) const;

	/**
	 * @return elements
	 */
	FormalRTEElement < SymbolType, RankType > & getLeftElement ( );
	FormalRTEElement < SymbolType, RankType > & getRightElement ( );

	/**
	 * @param element to append
	 */
	void setLeftElement ( const FormalRTEElement < SymbolType, RankType > & element );
	void setLeftElement ( FormalRTEElement < SymbolType, RankType > && element );

	/**
	 * @param element to append
	 */
	void setRightElement ( const FormalRTEElement < SymbolType, RankType > & element );
	void setRightElement ( FormalRTEElement < SymbolType, RankType > && element );

	virtual int compare ( const FormalRTEElement < SymbolType, RankType > & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const FormalRTEAlternation & ) const;

	/**
	 * @copydoc FormalRTEElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;
};

template < class SymbolType, class RankType >
FormalRTEAlternation < SymbolType, RankType >::FormalRTEAlternation ( FormalRTEElement < SymbolType, RankType > && left, FormalRTEElement < SymbolType, RankType > && right ) : std::BinaryNode < std::smart_ptr < FormalRTEElement < SymbolType, RankType > >, std::smart_ptr < FormalRTEElement < SymbolType, RankType > >, FormalRTEAlternation < SymbolType, RankType > > ( std::smart_ptr < FormalRTEElement < SymbolType, RankType > > ( std::move ( left ).plunder ( ) ), std::smart_ptr < FormalRTEElement < SymbolType, RankType > > ( std::move ( right ).plunder ( ) ) ) {
}

template < class SymbolType, class RankType >
FormalRTEAlternation < SymbolType, RankType >::FormalRTEAlternation ( const FormalRTEElement < SymbolType, RankType > & left, const FormalRTEElement < SymbolType, RankType > & right ) : FormalRTEAlternation ( std::move_copy ( left ), std::move_copy ( right ) ) {
}

template < class SymbolType, class RankType >
const FormalRTEElement < SymbolType, RankType > & FormalRTEAlternation < SymbolType, RankType >::getLeftElement ( ) const {
	return * this->getLeft ( );
}

template < class SymbolType, class RankType >
const FormalRTEElement < SymbolType, RankType > & FormalRTEAlternation < SymbolType, RankType >::getRightElement ( ) const {
	return * this->getRight ( );
}

template < class SymbolType, class RankType >
FormalRTEElement < SymbolType, RankType > & FormalRTEAlternation < SymbolType, RankType >::getLeftElement ( ) {
	return * this->getLeft ( );
}

template < class SymbolType, class RankType >
FormalRTEElement < SymbolType, RankType > & FormalRTEAlternation < SymbolType, RankType >::getRightElement ( ) {
	return * this->getRight ( );
}

template < class SymbolType, class RankType >
void FormalRTEAlternation < SymbolType, RankType >::setLeftElement ( const FormalRTEElement < SymbolType, RankType > & element ) {
	setLeftElement ( std::move_copy ( element ) );
}

template < class SymbolType, class RankType >
void FormalRTEAlternation < SymbolType, RankType >::setLeftElement ( FormalRTEElement < SymbolType, RankType > && element ) {
	setLeft ( std::smart_ptr < FormalRTEElement < SymbolType, RankType > > ( std::move ( element ).plunder ( ) ) );
}

template < class SymbolType, class RankType >
void FormalRTEAlternation < SymbolType, RankType >::setRightElement ( const FormalRTEElement < SymbolType, RankType > & element ) {
	setRightElement ( std::move_copy ( element ) );
}

template < class SymbolType, class RankType >
void FormalRTEAlternation < SymbolType, RankType >::setRightElement ( FormalRTEElement < SymbolType, RankType > && element ) {
	setRight ( std::smart_ptr < FormalRTEElement < SymbolType, RankType > > ( std::move ( element ).plunder ( ) ) );
}

template < class SymbolType, class RankType >
FormalRTEAlternation < SymbolType, RankType > * FormalRTEAlternation < SymbolType, RankType >::clone ( ) const {
	return new FormalRTEAlternation ( * this );
}

template < class SymbolType, class RankType >
FormalRTEAlternation < SymbolType, RankType > * FormalRTEAlternation < SymbolType, RankType >::plunder ( ) && {
	return new FormalRTEAlternation ( std::move ( * this ) );
}

template < class SymbolType, class RankType >
int FormalRTEAlternation < SymbolType, RankType >::compare ( const FormalRTEAlternation & other ) const {
	int res = getLeftElement ( ).compare ( other.getLeftElement ( ) );

	if ( res == 0 ) res = getRightElement ( ).compare ( other.getRightElement ( ) );

	return res;
}

template < class SymbolType, class RankType >
void FormalRTEAlternation < SymbolType, RankType >::operator >>( std::ostream & out ) const {
	out << "(FormalRTEAlternation";
	out << " " << getLeftElement ( );
	out << " " << getRightElement ( );
	out << ")";
}

template < class SymbolType, class RankType >
bool FormalRTEAlternation < SymbolType, RankType >::testSymbol ( const std::ranked_symbol < SymbolType, RankType > & symbol ) const {
	if ( getLeftElement ( ).testSymbol ( symbol ) ) return true;

	if ( getRightElement ( ).testSymbol ( symbol ) ) return true;

	return false;
}

template < class SymbolType, class RankType >
void FormalRTEAlternation < SymbolType, RankType >::computeMinimalAlphabet ( std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetF, std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetK ) const {
	getLeftElement ( ).computeMinimalAlphabet ( alphabetF, alphabetK );
	getRightElement ( ).computeMinimalAlphabet ( alphabetF, alphabetK );
}

template < class SymbolType, class RankType >
bool FormalRTEAlternation < SymbolType, RankType >::checkAlphabet ( const std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetF, const std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetK ) const {
	return getLeftElement ( ).checkAlphabet ( alphabetF, alphabetK ) && getRightElement ( ).checkAlphabet ( alphabetF, alphabetK );
}

template < class SymbolType, class RankType >
FormalRTEAlternation < SymbolType, RankType >::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace rte */

#endif /* FORMAL_RTE_ALTERNATION_H_ */
