/*
 * FormalRTEIteration.h
 *
 *  Created on: Feb 20, 2016
 *      Author: Tomas Pecka
 */

#ifndef FORMAL_RTE_ITERATION_H_
#define FORMAL_RTE_ITERATION_H_

#include "FormalRTEElement.h"
#include "FormalRTESymbolSubst.h"

#include <exception/CommonException.h>

#include <sstream>

namespace rte {

/**
 * Represents iteration operator in the regular expression. Contains one FormalRTEElement
 * as operand.
 */
template < class SymbolType, class RankType >
class FormalRTEIteration : public FormalRTEElement < SymbolType, RankType >, public std::UnaryNode < std::smart_ptr < FormalRTEElement < SymbolType, RankType > >, std::smart_ptr < const FormalRTEElement < SymbolType, RankType > >, FormalRTEIteration < SymbolType, RankType > > {
protected:
	std::smart_ptr < FormalRTESymbolSubst < SymbolType, RankType > > m_substitutionSymbol;

public:
	void accept ( typename FormalRTEElement < SymbolType, RankType >::Visitor & visitor ) const {
		visitor.visit ( * this );
	}

	explicit FormalRTEIteration ( FormalRTEElement < SymbolType, RankType > &&, FormalRTESymbolSubst < SymbolType, RankType > );
	explicit FormalRTEIteration ( const FormalRTEElement < SymbolType, RankType > &, FormalRTESymbolSubst < SymbolType, RankType > );

	/**
	 * @copydoc FormalRTEElement::clone() const
	 */
	virtual FormalRTEIteration < SymbolType, RankType > * clone ( ) const;

	/**
	 * @copydoc FormalRTEElement::plunder() const
	 */
	virtual FormalRTEIteration < SymbolType, RankType > * plunder ( ) &&;

	/**
	 * @copydoc FormalRTEElement::testSymbol() const
	 */
	virtual bool testSymbol ( const std::ranked_symbol < SymbolType, RankType > & symbol ) const;

	/**
	 * @copydoc FormalRTEElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetF, std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetK ) const;

	/**
	 * @copydoc FormalRTEElement::checkAlphabet()
	 */
	virtual bool checkAlphabet ( const std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetF, const std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetK ) const;

	/**
	 * @return element
	 */
	const FormalRTEElement < SymbolType, RankType > & getElement ( ) const;
	const FormalRTESymbolSubst < SymbolType, RankType > & getSubstitutionSymbol ( ) const;

	/**
	 * @return element
	 */
	FormalRTEElement < SymbolType, RankType > & getElement ( );
	FormalRTESymbolSubst < SymbolType, RankType > & getSubstitutionSymbol ( );

	/**
	 * @param element to iterate
	 */
	void setElement ( const FormalRTEElement < SymbolType, RankType > & element );
	void setElement ( FormalRTEElement < SymbolType, RankType > && element );

	void setSubstitutionSymbol ( FormalRTESymbolSubst < SymbolType, RankType > element );

	virtual int compare ( const FormalRTEElement < SymbolType, RankType > & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const FormalRTEIteration & ) const;

	/**
	 * @copydoc FormalRTEElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;
};

template < class SymbolType, class RankType >
FormalRTEIteration < SymbolType, RankType >::FormalRTEIteration ( FormalRTEElement < SymbolType, RankType > && element, FormalRTESymbolSubst < SymbolType, RankType > substitutionSymbol ) : std::UnaryNode < std::smart_ptr < FormalRTEElement < SymbolType, RankType > >, std::smart_ptr < const FormalRTEElement < SymbolType, RankType > >, FormalRTEIteration < SymbolType, RankType > > ( std::smart_ptr < FormalRTEElement < SymbolType, RankType > > ( std::move ( element ).plunder ( ) ) ), m_substitutionSymbol ( std::smart_ptr < FormalRTESymbolSubst < SymbolType, RankType > > ( std::move ( substitutionSymbol ).plunder ( ) ) ) {
}

template < class SymbolType, class RankType >
FormalRTEIteration < SymbolType, RankType >::FormalRTEIteration ( const FormalRTEElement < SymbolType, RankType > & element, FormalRTESymbolSubst < SymbolType, RankType > substitutionSymbol ) : FormalRTEIteration ( std::move_copy ( element ), std::move ( substitutionSymbol ) ) {
}

template < class SymbolType, class RankType >
const FormalRTEElement < SymbolType, RankType > & FormalRTEIteration < SymbolType, RankType >::getElement ( ) const {
	return * this->getChild ( );
}

template < class SymbolType, class RankType >
FormalRTEElement < SymbolType, RankType > & FormalRTEIteration < SymbolType, RankType >::getElement ( ) {
	return * this->getChild ( );
}

template < class SymbolType, class RankType >
const FormalRTESymbolSubst < SymbolType, RankType > & FormalRTEIteration < SymbolType, RankType >::getSubstitutionSymbol ( ) const {
	return * m_substitutionSymbol;
}

template < class SymbolType, class RankType >
FormalRTESymbolSubst < SymbolType, RankType > & FormalRTEIteration < SymbolType, RankType >::getSubstitutionSymbol ( ) {
	return * m_substitutionSymbol;
}

template < class SymbolType, class RankType >
void FormalRTEIteration < SymbolType, RankType >::setElement ( const FormalRTEElement < SymbolType, RankType > & element ) {
	setElement ( std::move_copy ( element ) );
}

template < class SymbolType, class RankType >
void FormalRTEIteration < SymbolType, RankType >::setElement ( FormalRTEElement < SymbolType, RankType > && elementParam ) {
	setChild ( std::smart_ptr < FormalRTEElement < SymbolType, RankType > > ( std::move ( elementParam ).plunder ( ) ) );
}

template < class SymbolType, class RankType >
void FormalRTEIteration < SymbolType, RankType >::setSubstitutionSymbol ( FormalRTESymbolSubst < SymbolType, RankType > symbol ) {
	m_substitutionSymbol = std::smart_ptr < FormalRTESymbolSubst < SymbolType, RankType > > ( std::move ( symbol ).plunder ( ) );
}

template < class SymbolType, class RankType >
FormalRTEIteration < SymbolType, RankType > * FormalRTEIteration < SymbolType, RankType >::clone ( ) const {
	return new FormalRTEIteration ( * this );
}

template < class SymbolType, class RankType >
FormalRTEIteration < SymbolType, RankType > * FormalRTEIteration < SymbolType, RankType >::plunder ( ) && {
	return new FormalRTEIteration ( std::move ( * this ) );
}

template < class SymbolType, class RankType >
int FormalRTEIteration < SymbolType, RankType >::compare ( const FormalRTEIteration & other ) const {
	auto first = std::tie ( getElement ( ), getSubstitutionSymbol ( ) );
	auto second = std::tie ( other.getElement ( ), other.getSubstitutionSymbol ( ) );

	std::compare < decltype ( first ) > comp;

	return comp ( first, second );
}

template < class SymbolType, class RankType >
void FormalRTEIteration < SymbolType, RankType >::operator >>( std::ostream & out ) const {
	out << "(RTEFormalRTEIteration " << * m_substitutionSymbol << " " << getElement ( ) << ")";
}

template < class SymbolType, class RankType >
bool FormalRTEIteration < SymbolType, RankType >::testSymbol ( const std::ranked_symbol < SymbolType, RankType > & symbol ) const {
	return this->getChild ( )->testSymbol ( symbol );
}

template < class SymbolType, class RankType >
void FormalRTEIteration < SymbolType, RankType >::computeMinimalAlphabet ( std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetF, std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetK ) const {
	alphabetK.insert ( m_substitutionSymbol->getSymbol ( ) );
	this->getChild ( )->computeMinimalAlphabet ( alphabetF, alphabetK );
}

template < class SymbolType, class RankType >
bool FormalRTEIteration < SymbolType, RankType >::checkAlphabet ( const std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetF, const std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetK ) const {
	return alphabetK.count ( getSubstitutionSymbol ( ).getSymbol ( ) ) > 0 && getElement ( ).checkAlphabet ( alphabetF, alphabetK );
}

template < class SymbolType, class RankType >
FormalRTEIteration < SymbolType, RankType >::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace rte */
#endif /* FORMAL_RTE_ITERATION_H_ */
