/*
 * FormalRTESymbolSubst.h
 *
 *  Created on: Feb 20, 2016
 *      Author: Tomas Pecka
 */

#ifndef FORMAL_RTE_SUBST_SYMBOL_H_
#define FORMAL_RTE_SUBST_SYMBOL_H_

#include "FormalRTESymbol.h"

#include <exception/CommonException.h>
#include <sstream>

namespace rte {

/**
 * Represents constant symbol (element of alphabet K) in the regular expression. Contains name of the symbol.
 */
template < class SymbolType, class RankType >
class FormalRTESymbolSubst : public FormalRTESymbol < SymbolType, RankType >, public std::NullaryNode < FormalRTESymbolSubst < SymbolType, RankType > > {
public:
	void accept ( typename FormalRTEElement < SymbolType, RankType >::Visitor & visitor ) const {
		visitor.visit ( * this );
	}

	explicit FormalRTESymbolSubst ( std::ranked_symbol < SymbolType, RankType > symbol );

	/**
	 * @copydoc FormalRTEElement::clone() const
	 */
	virtual FormalRTESymbolSubst < SymbolType, RankType > * clone ( ) const;

	/**
	 * @copydoc FormalRTEElement::plunder() const
	 */
	virtual FormalRTESymbolSubst < SymbolType, RankType > * plunder ( ) &&;

	/**
	 * @copydoc FormalRTEElement::testSymbol() const
	 */
	virtual bool testSymbol ( const std::ranked_symbol < SymbolType, RankType > & symbol ) const;

	/**
	 * @copydoc FormalRTEElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet ( std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetF, std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetK ) const;

	/**
	 * @copydoc FormalRTEElement::checkAlphabet()
	 */
	virtual bool checkAlphabet ( const std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetF, const std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetK ) const;

	virtual int compare ( const FormalRTESymbolSubst & ) const;

	virtual int compare ( const FormalRTEElement < SymbolType, RankType > & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	/**
	 * @copydoc FormalRTEElement::operator>>() const
	 */
	virtual void operator >>( std::ostream & out ) const;

	/**
	 * Returns the string representation of RTE Substitution Symbol.
	 */
	virtual explicit operator std::string ( ) const;
};

template < class SymbolType, class RankType >
FormalRTESymbolSubst < SymbolType, RankType >::FormalRTESymbolSubst ( std::ranked_symbol < SymbolType, RankType > symbol ) : FormalRTESymbol < SymbolType, RankType > ( symbol ) {
}

template < class SymbolType, class RankType >
FormalRTESymbolSubst < SymbolType, RankType > * FormalRTESymbolSubst < SymbolType, RankType >::clone ( ) const {
	return new FormalRTESymbolSubst ( * this );
}

template < class SymbolType, class RankType >
FormalRTESymbolSubst < SymbolType, RankType > * FormalRTESymbolSubst < SymbolType, RankType >::plunder ( ) && {
	return new FormalRTESymbolSubst ( std::move ( * this ) );
}

template < class SymbolType, class RankType >
int FormalRTESymbolSubst < SymbolType, RankType >::compare ( const FormalRTESymbolSubst & other ) const {
	std::compare < decltype ( this->getSymbol ( ) ) > comp;

	return comp ( this->getSymbol ( ), other.getSymbol ( ) );
}

template < class SymbolType, class RankType >
void FormalRTESymbolSubst < SymbolType, RankType >::operator >>( std::ostream & out ) const {
	out << "(FormalRTESymbolSubst " << " symbol = " << this->getSymbol ( ) << "})";
}

template < class SymbolType, class RankType >
bool FormalRTESymbolSubst < SymbolType, RankType >::testSymbol ( const std::ranked_symbol < SymbolType, RankType > & symbol ) const {
	return symbol == this->getSymbol ( );
}

template < class SymbolType, class RankType >
void FormalRTESymbolSubst < SymbolType, RankType >::computeMinimalAlphabet ( std::set < std::ranked_symbol < SymbolType, RankType > > & /* alphabetF */, std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetK ) const {
	alphabetK.insert ( this->getSymbol ( ) );
}

template < class SymbolType, class RankType >
bool FormalRTESymbolSubst < SymbolType, RankType >::checkAlphabet ( const std::set < std::ranked_symbol < SymbolType, RankType > > & /* alphabetF */, const std::set < std::ranked_symbol < SymbolType, RankType > > & alphabetK ) const {
	return alphabetK.count ( this->getSymbol ( ) ) > 0;
}

template < class SymbolType, class RankType >
FormalRTESymbolSubst < SymbolType, RankType >::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace rte */

#endif /* FORMAL_RTE_SUBST_SYMBOL_H_ */
