/*
 * TreeAuxiliary.h
 *
 *  Created on: May 5, 2016
 *      Author: Jan Travnicek
 */

#ifndef TREE_AUXILIARY_H_
#define TREE_AUXILIARY_H_

#include <set>
#include <tree>
#include <algorithm>
#include <alphabet/ranked_symbol.hpp>

namespace tree {

/**
 * Parser used to get tree from XML parsed into list of Tokens.
 */
class TreeAuxiliary {
public:
	template < class SymbolType, class RankType >
	static std::set < SymbolType > unrankSymbols ( const std::set < std::ranked_symbol < SymbolType, RankType > > & alphabet );
	template < class SymbolType, class RankType >
	static std::set < std::ranked_symbol < SymbolType, RankType > > computeBars ( const std::set < std::ranked_symbol < SymbolType, RankType > > & alphabet, const SymbolType & barBase );

	template < class SymbolType, class RankType >
	static std::tree < std::ranked_symbol < SymbolType, RankType > > toRankedTree ( const std::tree < SymbolType > & from );
	template < class SymbolType, class RankType >
	static std::tree < SymbolType > toUnrankedTree ( const std::tree < std::ranked_symbol < SymbolType, RankType > > & from );

};

template < class SymbolType, class RankType >
std::set < SymbolType > TreeAuxiliary::unrankSymbols ( const std::set < std::ranked_symbol < SymbolType, RankType > > & alphabet ) {
	return std::transform < SymbolType > ( alphabet, [&] ( const std::ranked_symbol < SymbolType, RankType > & symbol ) {
			return symbol.getSymbol ( );
	} );
}

template < class SymbolType, class RankType >
std::set < std::ranked_symbol < SymbolType, RankType > > TreeAuxiliary::computeBars ( const std::set < std::ranked_symbol < SymbolType, RankType > > & alphabet, const SymbolType & barBase ) {
	return std::transform < std::ranked_symbol < SymbolType, RankType > > ( alphabet, [&] ( const std::ranked_symbol < SymbolType, RankType > & symbol ) {
			return std::ranked_symbol < SymbolType, RankType > ( barBase, symbol.getRank ( ) );
	} );
}

template < class SymbolType, class RankType >
std::tree < std::ranked_symbol < SymbolType, RankType > > TreeAuxiliary::toRankedTree ( const std::tree < SymbolType > & tree ) {
	std::vector < std::tree < std::ranked_symbol < SymbolType, RankType > > > children;

	for ( const std::tree < SymbolType > & child : tree ) {
		children.push_back ( toRankedTree < SymbolType, RankType > ( child ) );
	}

	unsigned size = children.size ( );
	return std::tree < std::ranked_symbol < SymbolType, RankType > > ( std::ranked_symbol < SymbolType, RankType > ( tree.getData ( ), size ), std::move ( children ) );
}

template < class SymbolType, class RankType >
std::tree < SymbolType > TreeAuxiliary::toUnrankedTree ( const std::tree < std::ranked_symbol < SymbolType, RankType > > & tree ) {
	std::vector < std::tree < SymbolType > > children;

	for ( const std::tree < std::ranked_symbol < SymbolType, RankType > > & child : tree ) {
		children.push_back ( toUnrankedTree < SymbolType, RankType > ( child ) );
	}

	return std::tree < SymbolType > ( tree.getData ( ).getSymbol ( ), std::move ( children ) );
}

} /* namespace tree */

#endif /* TREE_AUXILIARY_H_ */
