/*
 * UndirectedEdge.h
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 */

#ifndef UNDIRECTED_EDGE_H_
#define UNDIRECTED_EDGE_H_

#include "../common/Macros.h"
#include "../common/Node.h"

namespace graph {

class UndirectedEdge {
	Node m_first;
	Node m_second;
	label::Label m_name;

	void dump(std::ostream & os) const;

public:
	/// should be used only to create variable, to which will be assigned soon
	UndirectedEdge() : m_first(), m_second(), m_name(label::Label("uninitialized")) {}

	explicit UndirectedEdge(Node first, Node second);
	explicit UndirectedEdge(Node first, Node second, label::Label name);
	explicit UndirectedEdge(Node first, Node second, int number);
	explicit UndirectedEdge(Node first, Node second, char character);
	explicit UndirectedEdge(Node first, Node second, std::string name);

	UndirectedEdge * clone() const;
	UndirectedEdge * plunder() &&;

	const Node & getFirstNode() const;
	const Node & getSecondNode() const;
	const label::Label & getName() const;

	bool operator < (const UndirectedEdge & other) const;
	bool operator > (const UndirectedEdge & other) const;
	bool operator == (const UndirectedEdge & other) const;
	bool operator != (const UndirectedEdge & other) const;
	int compare(const UndirectedEdge & other) const;

	friend std::ostream & operator<<(std::ostream & out, const UndirectedEdge & node);
	explicit operator std::string() const;
};

} // namespace graph

namespace std {

template<>
struct compare<graph::UndirectedEdge> {
	int operator()(const graph::UndirectedEdge& first, const graph::UndirectedEdge& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

GRAPH_DEFINE_STD_HASH(UndirectedEdge)

#endif // UNDIRECTED_EDGE_H_
