/*
 * SuffixTrieNodeFinalMark.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Stepan Plachy
 */

#ifndef SUFFIX_TRIE_NODE_FINAL_MARK_H_
#define SUFFIX_TRIE_NODE_FINAL_MARK_H_

#include <common/DefaultSymbolType.h>
#include <map>
#include <set>

namespace indexes {

class SuffixTrieNodeTerminatingSymbol;
class SuffixTrieFinalMark;

/**
 * Represents a node in the ranked indexes. Contains name of the symbol.
 */
class SuffixTrieNodeFinalMark {
protected:
	std::map < DefaultSymbolType, SuffixTrieNodeFinalMark * > m_children;

	bool m_finalMark;

	SuffixTrieNodeFinalMark * parent;

	/**
	 * Parent indexes contanining this instance of RankedTree
	 */
	const SuffixTrieFinalMark * parentTree;

	/**
	 * @copydoc SuffixTrieNodeFinalMark::attachTree()
	 */
	bool attachTree ( const SuffixTrieFinalMark * indexes );

public:
	explicit SuffixTrieNodeFinalMark ( std::map < DefaultSymbolType, SuffixTrieNodeFinalMark * > children, bool finalMark );

	explicit SuffixTrieNodeFinalMark ( const SuffixTrieNodeTerminatingSymbol & other, const DefaultSymbolType & terminatingSymbol );

	SuffixTrieNodeFinalMark ( const SuffixTrieNodeFinalMark & other );
	SuffixTrieNodeFinalMark ( SuffixTrieNodeFinalMark && other ) noexcept;
	SuffixTrieNodeFinalMark & operator =( const SuffixTrieNodeFinalMark & other );
	SuffixTrieNodeFinalMark & operator =( SuffixTrieNodeFinalMark && other ) noexcept;
	~SuffixTrieNodeFinalMark ( ) noexcept;

	/**
	 * @copydoc SuffixTrieNodeFinalMark::clone() const
	 */
	SuffixTrieNodeFinalMark * clone ( ) const;

	/**
	 * @copydoc SuffixTrieNodeFinalMark::plunder() const
	 */
	SuffixTrieNodeFinalMark * plunder ( ) &&;

	/**
	 * @copydoc SuffixTrieNodeFinalMark::computeMinimalAlphabet()
	 */
	std::set < DefaultSymbolType > computeMinimalAlphabet ( ) const;

	/**
	 * @copydoc SuffixTrieNodeFinalMark::computeMinimalAlphabet()
	 */
	void computeMinimalAlphabet ( std::set < DefaultSymbolType > & alphabet ) const;

	/**
	 * @return children
	 */
	const std::map < const DefaultSymbolType, const SuffixTrieNodeFinalMark * > & getChildren ( ) const;

	bool getFinalMark ( ) const;

	void setFinalMark ( bool newFinalMark );

	/**
	 * @return children
	 */
	const std::map < DefaultSymbolType, SuffixTrieNodeFinalMark * > & getChildren ( );

	SuffixTrieNodeFinalMark & getChild ( const DefaultSymbolType & symbol );

	const SuffixTrieNodeFinalMark & getChild ( const DefaultSymbolType & symbol ) const;

	bool hasChild ( const DefaultSymbolType & symbol ) const;

	SuffixTrieNodeFinalMark & addChild ( DefaultSymbolType symbol, SuffixTrieNodeFinalMark node );

	SuffixTrieNodeFinalMark * getParent ( );

	const SuffixTrieNodeFinalMark * getParent ( ) const;

	void swap ( SuffixTrieNodeFinalMark & other );

	int compare ( const SuffixTrieNodeFinalMark & ) const;

	/**
	 * @copydoc SuffixTrieNodeFinalMark::testSymbol() const
	 */
	bool testSymbol ( const DefaultSymbolType & symbol ) const;

	/**
	 * @copydoc SuffixTrieNodeFinalMark::operator>>() const
	 */
	void operator >>( std::ostream & out ) const;

	friend std::ostream & operator <<( std::ostream &, const SuffixTrieNodeFinalMark & node );

	explicit operator std::string ( ) const;

	friend class SuffixTrieFinalMark;
};

} /* namespace indexes */

namespace std {

template < >
struct compare < indexes::SuffixTrieNodeFinalMark > {
	int operator ()( const indexes::SuffixTrieNodeFinalMark & first, const indexes::SuffixTrieNodeFinalMark & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

#endif /* SUFFIX_TRIE_NODE_FINAL_MARK_H_ */
