/*
 * SymbolFromStringParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "SymbolFromStringParser.h"
#include "exception/CommonException.h"
#include "alphabet/BlankSymbol.h"
#include "alphabet/BottomOfTheStackSymbol.h"
#include "alphabet/EndSymbol.h"
#include "alphabet/StartSymbol.h"
#include "alphabet/LabeledSymbol.h"
#include <alphabet/Symbol.h>

#include "../StringApi.hpp"

namespace alphabet {

Symbol SymbolFromStringParser::parseSymbol(std::istream& input) {
	return parseSymbol(input, std::set<FEATURES>({FEATURES::LABELED, FEATURES::BLANK, FEATURES::BOTTOM, FEATURES::END, FEATURES::START}));
}

Symbol SymbolFromStringParser::parseSymbol(std::istream& input, const std::set<FEATURES>& features) {
	SymbolFromStringLexer::Token token = SymbolFromStringLexer::next(input);
	switch(token.type) {
	case SymbolFromStringLexer::TokenType::BLANK:
		if(!features.count(FEATURES::BLANK)) throw exception::CommonException("Disabled formalism BlankSymbol");
		return Symbol(BlankSymbol());
	case SymbolFromStringLexer::TokenType::BOTTOM:
		if(!features.count(FEATURES::BOTTOM)) throw exception::CommonException("Disabled formalism BottomSymbol");
		return Symbol(BottomOfTheStackSymbol());
	case SymbolFromStringLexer::TokenType::END:
		if(!features.count(FEATURES::END)) throw exception::CommonException("Disabled formalism EndSymbol");
		return Symbol(EndSymbol());
	case SymbolFromStringLexer::TokenType::START:
		if(!features.count(FEATURES::START)) throw exception::CommonException("Disabled formalism StartSymbol");
		return Symbol(StartSymbol());
	case SymbolFromStringLexer::TokenType::ERROR:
		if(!features.count(FEATURES::LABELED)) throw exception::CommonException("Disabled formalism LabeledSymbol");
		SymbolFromStringLexer::putback(input, token);
		return Symbol(LabeledSymbol(alib::stringApi<label::Label>::parse(input)));
	case SymbolFromStringLexer::TokenType::TEOF:
		throw exception::CommonException("Unexpected end of file");
	}
	throw exception::CommonException("Formalism not recognised (token = \"" + token.value + "\")");
}

} /* namespace alphabet */
