/*
 * SymbolToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include <object/ObjectToStringComposer.h>

#include "SymbolToStringComposer.h"
#include "alphabet/LabeledSymbol.h"
#include "alphabet/StartSymbol.h"
#include "alphabet/EndSymbol.h"
#include "alphabet/BottomOfTheStackSymbol.h"
#include "alphabet/BlankSymbol.h"
#include "alphabet/BarSymbol.h"
#include "alphabet/VariablesBarSymbol.h"
#include "alphabet/SubtreeWildcardSymbol.h"
#include "alphabet/NonlinearVariableSymbol.h"
#include "alphabet/RankedSymbol.h"
#include "alphabet/SymbolPairSymbol.h"
#include "alphabet/SymbolSetSymbol.h"
#include "alphabet/UniqueSymbol.h"

#include "../primitive/PrimitiveStringApi.hpp"

namespace alphabet {

void SymbolToStringComposer::compose(std::ostream& out, const LabeledSymbol& symbol) {
	alib::stringApi<label::Label>::compose(out, symbol.getLabel());
}

auto SymbolToStringComposerLabeledSymbol = SymbolToStringComposer::RegistratorWrapper<void, LabeledSymbol>(SymbolToStringComposer::compose);
auto ObjectToStringComposerLabeledSymbol = alib::ObjectBaseDispatcher::RegistratorWrapper<void, LabeledSymbol >( SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const BlankSymbol&) {
	out << "#B";
}

auto SymbolToStringComposerBlankSymbol = SymbolToStringComposer::RegistratorWrapper<void, BlankSymbol>(SymbolToStringComposer::compose);
auto ObjectToStringComposerBlankSymbol = alib::ObjectBaseDispatcher::RegistratorWrapper<void, BlankSymbol >( SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const BottomOfTheStackSymbol&) {
	out << "#T";
}

auto SymbolToStringComposerBottomOfTheStackSymbol = SymbolToStringComposer::RegistratorWrapper<void, BottomOfTheStackSymbol>(SymbolToStringComposer::compose);
auto ObjectToStringComposerBottomOfTheStackSymbol = alib::ObjectBaseDispatcher::RegistratorWrapper<void, BottomOfTheStackSymbol >( SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const EndSymbol&) {
	out << "#$";
}

auto SymbolToStringComposerEndSymbol = SymbolToStringComposer::RegistratorWrapper<void, EndSymbol>(SymbolToStringComposer::compose);
auto ObjectToStringComposerEndSymbol = alib::ObjectBaseDispatcher::RegistratorWrapper<void, EndSymbol >( SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const StartSymbol&) {
	out << "#^";
}

auto SymbolToStringComposerStartSymbol = SymbolToStringComposer::RegistratorWrapper<void, StartSymbol>(SymbolToStringComposer::compose);
auto ObjectToStringComposerStartSymbol = alib::ObjectBaseDispatcher::RegistratorWrapper<void, StartSymbol >( SymbolToStringComposer::compose);

template < class SymbolType, class RankType >
void SymbolToStringComposer::compose(std::ostream& out, const RankedSymbol < SymbolType, RankType > & symbol) {
	alib::ObjectToStringComposer::compose(out, symbol.getSymbol());
	alib::ObjectToStringComposer::compose(out, symbol.getRank());
}

auto SymbolToStringComposerRankedSymbol = SymbolToStringComposer::RegistratorWrapper<void, RankedSymbol < > >(SymbolToStringComposer::compose);
auto ObjectToStringComposerRankedSymbol = alib::ObjectBaseDispatcher::RegistratorWrapper<void, RankedSymbol < > >( SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const BarSymbol&) {
	out << "#|";
}

auto SymbolToStringComposerBarSymbol = SymbolToStringComposer::RegistratorWrapper<void, BarSymbol>(SymbolToStringComposer::compose);
auto ObjectToStringComposerBarSymbol = alib::ObjectBaseDispatcher::RegistratorWrapper<void, BarSymbol >( SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const VariablesBarSymbol&) {
	out << "#/";
}

auto SymbolToStringComposerVariablesBarSymbol = SymbolToStringComposer::RegistratorWrapper<void, VariablesBarSymbol>(SymbolToStringComposer::compose);
auto ObjectToStringComposerVariablesBarSymbol = alib::ObjectBaseDispatcher::RegistratorWrapper<void, VariablesBarSymbol >( SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const SubtreeWildcardSymbol&) {
	out << "#S";
}

auto SymbolToStringComposerSubtreeWildcardSymbol = SymbolToStringComposer::RegistratorWrapper<void, SubtreeWildcardSymbol>(SymbolToStringComposer::compose);
auto ObjectToStringComposerSubtreeWildcardSymbol = alib::ObjectBaseDispatcher::RegistratorWrapper<void, SubtreeWildcardSymbol >( SymbolToStringComposer::compose);

template < class SymbolType >
void SymbolToStringComposer::compose(std::ostream& out, const NonlinearVariableSymbol < SymbolType > & symbol ) {
	out << "$";
	alib::ObjectToStringComposer::compose(out, symbol.getSymbol());
}

auto SymbolToStringComposerNonlinearVariableSymbol = SymbolToStringComposer::RegistratorWrapper<void, NonlinearVariableSymbol < > >(SymbolToStringComposer::compose);
auto ObjectToStringComposerNonlinearVariableSymbol = alib::ObjectBaseDispatcher::RegistratorWrapper<void, NonlinearVariableSymbol < > >( SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const SymbolSetSymbol& symbol) {
	out << '[';
	bool first = true;
	for(const Symbol& innerSymbol : symbol.getData()) {
		if(!first)
			out << ", ";
		else
			first = false;
		dispatch(out, innerSymbol.getData());
	}
	out << ']';
}

auto SymbolToStringComposerSymbolSetSymbol = SymbolToStringComposer::RegistratorWrapper<void, SymbolSetSymbol>(SymbolToStringComposer::compose);
auto ObjectToStringComposerSymbolSetSymbol = alib::ObjectBaseDispatcher::RegistratorWrapper<void, SymbolSetSymbol >( SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const SymbolPairSymbol& symbol) {
	out << '<';
	dispatch(out, symbol.getData().first.getData());
	out << ", ";
	dispatch(out, symbol.getData().second.getData());
	out << '>';
}

auto SymbolToStringComposerSymbolPairSymbol = SymbolToStringComposer::RegistratorWrapper<void, SymbolPairSymbol>(SymbolToStringComposer::compose);
auto ObjectToStringComposerSymbolPairSymbol = alib::ObjectBaseDispatcher::RegistratorWrapper<void, SymbolPairSymbol >( SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const UniqueSymbol& symbol) {
	out << '<';
	dispatch(out, symbol.getSymbol().getData());
	out << ", ";
	alib::stringApi<primitive::Primitive>::compose(out, primitive::Primitive(symbol.getId()));
	out << '>';
}

auto SymbolToStringComposerUniqueSymbol = SymbolToStringComposer::RegistratorWrapper<void, UniqueSymbol>(SymbolToStringComposer::compose);
auto ObjectToStringComposerUniqueSymbol = alib::ObjectBaseDispatcher::RegistratorWrapper<void, UniqueSymbol >( SymbolToStringComposer::compose);

void SymbolToStringComposer::compose(std::ostream& out, const Symbol& symbol) {
	dispatch(out, symbol.getData());
}

auto ObjectToStringComposerSymbol = alib::WrapperBaseDispatcher::RegistratorWrapper<void, Symbol>(SymbolToStringComposer::compose);

} /* namespace alphabet */
