/*
 * AutomatonToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "AutomatonToStringComposer.h"
#include "exception/CommonException.h"
#include "automaton/FSM/DFA.h"
#include "automaton/FSM/NFA.h"
#include "automaton/FSM/MultiInitialStateNFA.h"
#include "automaton/FSM/EpsilonNFA.h"

namespace automaton {

template < class SymbolType, class StateType >
void AutomatonToStringComposer::compose(std::ostream& out, const DFA < SymbolType, StateType >& automaton) {
	out << "DFA";
	for(const auto& symbol : automaton.getInputAlphabet()) {
		out << " ";
		alib::ObjectToStringComposer::compose(out, symbol);
	}

	out << std::endl;

	for(const auto& state : automaton.getStates()) {
		if(automaton.getInitialState() == state) {
			out << ">";
		}
		if(automaton.getFinalStates().find(state) != automaton.getFinalStates().end()) {
			out << "<";
		}
		alib::ObjectToStringComposer::compose(out, state);

		composeTransitionsFromState(out, automaton, state);

		out << std::endl;
	}
}

auto AutomatonToStringComposerDFA = AutomatonToStringComposer::RegistratorWrapper<void, DFA < > >(AutomatonToStringComposer::compose);
auto ObjectToStringComposerDFA = alib::ObjectBaseDispatcher::RegistratorWrapper<void, DFA < > >( AutomatonToStringComposer::compose);

template < class SymbolType, class StateType >
void AutomatonToStringComposer::compose(std::ostream& out, const NFA < SymbolType, StateType > & automaton) {
	out << "NFA";
	for(const auto& symbol : automaton.getInputAlphabet()) {
		out << " ";
		alib::ObjectToStringComposer::compose(out, symbol);
	}

	out << std::endl;

	for(const auto& state : automaton.getStates()) {
		if(automaton.getInitialState() == state) {
			out << ">";
		}
		if(automaton.getFinalStates().find(state) != automaton.getFinalStates().end()) {
			out << "<";
		}
		alib::ObjectToStringComposer::compose(out, state);

		composeTransitionsFromState(out, automaton, state);

		out << std::endl;
	}
}

auto AutomatonToStringComposerNFA = AutomatonToStringComposer::RegistratorWrapper<void, NFA < > >(AutomatonToStringComposer::compose);
auto ObjectToStringComposerNFA = alib::ObjectBaseDispatcher::RegistratorWrapper<void, NFA < > >( AutomatonToStringComposer::compose);

template < class SymbolType, class StateType >
void AutomatonToStringComposer::compose(std::ostream& out, const MultiInitialStateNFA < SymbolType, StateType > & automaton) {
	out << "MISNFA";
	for(const auto& symbol : automaton.getInputAlphabet()) {
		out << " ";
		alib::ObjectToStringComposer::compose(out, symbol);
	}

	out << std::endl;

	for(const auto& state : automaton.getStates()) {
		if(automaton.getInitialStates().find(state) != automaton.getInitialStates().end()) {
			out << ">";
		}
		if(automaton.getFinalStates().find(state) != automaton.getFinalStates().end()) {
			out << "<";
		}
		alib::ObjectToStringComposer::compose(out, state);

		composeTransitionsFromState(out, automaton, state);

		out << std::endl;
	}
}

auto AutomatonToStringComposerMultiInitialStateNFA = AutomatonToStringComposer::RegistratorWrapper<void, MultiInitialStateNFA < > >(AutomatonToStringComposer::compose);
auto ObjectToStringComposerMultiInitialStateNFA = alib::ObjectBaseDispatcher::RegistratorWrapper<void, MultiInitialStateNFA < > >( AutomatonToStringComposer::compose);

template < class SymbolType, class EpsilonType, class StateType >
void AutomatonToStringComposer::compose(std::ostream& out, const EpsilonNFA < SymbolType, EpsilonType, StateType > & automaton) {
	out << "ENFA";
	for(const auto& symbol : automaton.getInputAlphabet()) {
		out << " ";
		alib::ObjectToStringComposer::compose(out, symbol);
	}

	out << " #E";
	out << std::endl;

	for(const auto& state : automaton.getStates()) {
		if(automaton.getInitialState() == state) {
			out << ">";
		}
		if(automaton.getFinalStates().find(state) != automaton.getFinalStates().end()) {
			out << "<";
		}
		alib::ObjectToStringComposer::compose(out, state);

		composeTransitionsFromState(out, automaton, state);

		out << std::endl;
	}
}

auto AutomatonToStringComposerEpsilonNFA = AutomatonToStringComposer::RegistratorWrapper<void, EpsilonNFA < > >(AutomatonToStringComposer::compose);
auto ObjectToStringComposerEpsilonNFA = alib::ObjectBaseDispatcher::RegistratorWrapper<void, EpsilonNFA < > >( AutomatonToStringComposer::compose);

void AutomatonToStringComposer::compose(std::ostream& output, const Automaton& automaton) {
	dispatch(output, automaton.getData());
}

auto ObjectToStringComposerAutomaton = alib::WrapperBaseDispatcher::RegistratorWrapper<void, Automaton>(AutomatonToStringComposer::compose);

} /* namespace automaton */
