/*
 * ObjectToStringComposer.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnciek
 */

#ifndef OBJECT_TO_STRING_COMPOSER_H_
#define OBJECT_TO_STRING_COMPOSER_H_

#include <ostream>
#include <core/multipleDispatch.hpp>
#include <object/Object.h>
#include <object/Void.h>
#include <container/ObjectsPair.h>
#include <container/ObjectsSet.h>

namespace alib {

class ObjectToStringComposer;

typedef std::SingleDispatchFirstStaticParam<ObjectToStringComposer, void, std::ostream&, const CommonBaseBase &> ObjectBaseDispatcher;
typedef std::SingleDispatchFirstStaticParam<ObjectToStringComposer, void, std::ostream&, const WrapperBaseBase &> WrapperBaseDispatcher;

/**
 * This class contains methods to print XML representation of string to the output stream.
 */
class ObjectToStringComposer : public ObjectBaseDispatcher, public WrapperBaseDispatcher {
public:
	/**
	 * Prints text representation of Object to the output stream.
	 * @param string Object to print
	 * @param out output stream to which print the String
	 */
	static void compose(std::ostream& output, const Object& object);
	static void compose(std::ostream& output, const Void& object);
	template < class T, class R >
	static void compose(std::ostream& output, const container::ObjectsPair < T, R > & object);
	template < class T >
	static void compose(std::ostream& output, const container::ObjectsSet < T > & object);

	static void compose(std::ostream& output, const CommonBaseBase& object);
	static void compose(std::ostream& output, const WrapperBaseBase& object);
};

} /* namespace alib */

#endif /* OBJECT_TO_STRING_COMPOSER_H_ */
