/*
 * PrimitiveToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include <object/ObjectToStringComposer.h>

#include "PrimitiveToStringComposer.h"
#include "primitive/Integer.h"
#include "primitive/String.h"
#include "primitive/Character.h"
#include "primitive/Unsigned.h"
#include "primitive/Bool.h"

namespace primitive {

void PrimitiveToStringComposer::compose(std::ostream& out, const String& primitive) {
	auto replace = [](std::string& str, const std::string& what, const std::string& with) {
		size_t index = 0;
		while((index = str.find(what, index)) != std::string::npos) {
			str.replace(index, what.length(), with);
			index += with.length();
		}
	};
	
	std::string tmp = primitive.getData();
	replace(tmp, "'", "\\'" );
	out << '\'' << tmp << '\'';
}

auto PrimitiveToStringComposerString = PrimitiveToStringComposer::RegistratorWrapper<void, String>(PrimitiveToStringComposer::compose);
auto ObjectToStringComposerString = alib::ObjectBaseDispatcher::RegistratorWrapper<void, String>(PrimitiveToStringComposer::compose);

void PrimitiveToStringComposer::compose(std::ostream& out, const Character& primitive) {
	out << primitive.getData();
}

auto PrimitiveToStringComposerCharacter = PrimitiveToStringComposer::RegistratorWrapper<void, Character>(PrimitiveToStringComposer::compose);
auto ObjectToStringComposerCharacter = alib::ObjectBaseDispatcher::RegistratorWrapper<void, Character>(PrimitiveToStringComposer::compose);

void PrimitiveToStringComposer::compose(std::ostream& out, const Integer& primitive) {
	out << primitive.getData();
}

auto PrimitiveToStringComposerInteger = PrimitiveToStringComposer::RegistratorWrapper<void, Integer>(PrimitiveToStringComposer::compose);
auto ObjectToStringComposerInteger = alib::ObjectBaseDispatcher::RegistratorWrapper<void, Integer>(PrimitiveToStringComposer::compose);

void PrimitiveToStringComposer::compose(std::ostream& out, const Unsigned& primitive) {
	out << primitive.getData();
}

auto PrimitiveToStringComposerUnsigned = PrimitiveToStringComposer::RegistratorWrapper<void, Unsigned>(PrimitiveToStringComposer::compose);
auto ObjectToStringComposerUnsigned = alib::ObjectBaseDispatcher::RegistratorWrapper<void, Unsigned>(PrimitiveToStringComposer::compose);

void PrimitiveToStringComposer::compose(std::ostream& out, const Bool& primitive) {
	out << (primitive.getData() ? "true" : "false");
}

auto PrimitiveToStringComposerBool = PrimitiveToStringComposer::RegistratorWrapper<void, Bool>(PrimitiveToStringComposer::compose);
auto ObjectToStringComposerBool = alib::ObjectBaseDispatcher::RegistratorWrapper<void, Bool>(PrimitiveToStringComposer::compose);

void PrimitiveToStringComposer::compose(std::ostream& out, const Primitive& primitive) {
	dispatch(out, primitive.getData());
}

auto ObjectToStringComposerPrimitive = alib::WrapperBaseDispatcher::RegistratorWrapper<void, Primitive>(PrimitiveToStringComposer::compose);

} /* namespace primitive */
