/*
 * StringToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include <object/ObjectToStringComposer.h>

#include "StringToStringComposer.h"
#include "string/CyclicString.h"
#include "string/LinearString.h"
#include "string/Epsilon.h"

namespace string {

template < class SymbolType >
void StringToStringComposer::compose ( std::ostream & out, const CyclicString < SymbolType > & string ) {
	out << "<";

	for ( const auto & symbol : string.getContent ( ) )
		alib::ObjectToStringComposer::compose ( out, symbol );

	out << ">";
}

auto StringToStringComposerCyclicString = StringToStringComposer::RegistratorWrapper < void, CyclicString < > > ( StringToStringComposer::compose );
auto ObjectToStringComposerCyclicString = alib::ObjectBaseDispatcher::RegistratorWrapper<void, CyclicString < > >( StringToStringComposer::compose);

template < class SymbolType >
void StringToStringComposer::compose ( std::ostream & out, const LinearString < SymbolType > & string ) {
	out << "\"";

	for ( const auto & symbol : string.getContent ( ) )
		alib::ObjectToStringComposer::compose ( out, symbol );

	out << "\"";
}

auto StringToStringComposerLinearString = StringToStringComposer::RegistratorWrapper < void, LinearString < > > ( StringToStringComposer::compose );
auto ObjectToStringComposerLinearString = alib::ObjectBaseDispatcher::RegistratorWrapper<void, LinearString < > >( StringToStringComposer::compose);

template < class SymbolType >
void StringToStringComposer::compose ( std::ostream & out, const Epsilon < SymbolType > & ) {
	out << "#E";
}

auto StringToStringComposerEpsilon = StringToStringComposer::RegistratorWrapper < void, Epsilon < > > ( StringToStringComposer::compose );
auto ObjectToStringComposerEpsilon = alib::ObjectBaseDispatcher::RegistratorWrapper<void, Epsilon < > >( StringToStringComposer::compose);

void StringToStringComposer::compose ( std::ostream & out, const String & string ) {
	dispatch ( out, string.getData ( ) );
}

auto ObjectToStringComposerString = alib::WrapperBaseDispatcher::RegistratorWrapper<void, String>(StringToStringComposer::compose);

} /* namespace string */
