/*
 * TreeToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "TreeToStringComposer.h"
#include <tree/ranked/RankedTree.h>
#include <tree/ranked/RankedPattern.h>
#include <tree/unranked/UnrankedTree.h>
#include <tree/unranked/UnrankedPattern.h>

namespace tree {

template < class SymbolType, class RankType >
void TreeToStringComposer::compose ( std::ostream & out, const RankedTree < SymbolType, RankType > & tree ) {
	compose ( out, tree.getContent ( ) );
}

auto StringToStringComposerRankedTree = TreeToStringComposer::RegistratorWrapper < void, RankedTree < > > ( TreeToStringComposer::compose );
auto ObjectToStringComposerRankedTree = alib::ObjectBaseDispatcher::RegistratorWrapper<void, RankedTree < > >(TreeToStringComposer::compose);

template < class SymbolType, class RankType >
void TreeToStringComposer::compose ( std::ostream & out, const RankedPattern < SymbolType, RankType > & tree ) {
	compose ( out, tree.getSubtreeWildcard ( ), tree.getContent ( ) );
}

auto StringToStringComposerRankedPattern = TreeToStringComposer::RegistratorWrapper < void, RankedPattern < > > ( TreeToStringComposer::compose );
auto ObjectToStringComposerRankedPattern = alib::ObjectBaseDispatcher::RegistratorWrapper<void, RankedPattern < > >(TreeToStringComposer::compose);

template < class SymbolType >
void TreeToStringComposer::compose ( std::ostream & out, const UnrankedTree < SymbolType > & tree ) {
	compose ( out, tree.getContent ( ) );
}

auto StringToStringComposerUnrankedTree = TreeToStringComposer::RegistratorWrapper < void, UnrankedTree < > > ( TreeToStringComposer::compose );
auto ObjectToStringComposerUnrankedTree = alib::ObjectBaseDispatcher::RegistratorWrapper<void, UnrankedTree < > >(TreeToStringComposer::compose);

template < class SymbolType >
void TreeToStringComposer::compose ( std::ostream & out, const UnrankedPattern < SymbolType > & tree ) {
	compose ( out, tree.getSubtreeWildcard ( ), tree.getContent ( ) );
}

auto StringToStringComposerUnrankedPattern = TreeToStringComposer::RegistratorWrapper < void, UnrankedPattern < > > ( TreeToStringComposer::compose );
auto ObjectToStringComposerUnrankedPattern = alib::ObjectBaseDispatcher::RegistratorWrapper<void, UnrankedPattern < > >(TreeToStringComposer::compose);

void TreeToStringComposer::compose ( std::ostream & out, const Tree & tree ) {
	dispatch ( out, tree.getData ( ) );
}

auto ObjectToStringComposerTree = alib::WrapperBaseDispatcher::RegistratorWrapper<void, Tree>(TreeToStringComposer::compose);

} /* namespace tree */
