/*
 * anormalize.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <sax/FromXMLParserHelper.h>

#include "exception/CommonException.h"
#include "factory/XmlDataFactory.hpp"
#include "string/simplify/NormalizeAlphabet.h"
#include "automaton/simplify/Normalize.h"
#include "automaton/simplify/SingleInitialState.h"
#include "grammar/convert/ToGrammarLeftRG.h"
#include "grammar/convert/ToGrammarRightRG.h"
#include "grammar/simplify/ToCNF.h"
#include "grammar/simplify/ToGNF.h"
#include "grammar/parsing/DeterministicLL1Grammar.h"

int main ( int argc, char * * argv ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "Automaton normalize binary", ' ', "0.01" );

		TCLAP::ValueArg < std::string > input ( "i", "input", "Input to normalize", false, "-", "file" );
		cmd.add ( input );

		std::vector < std::string > forms {
			"leftRG", "rightRG", "singleInitialState", "CNF", "GNF", "LL1"
		};
		TCLAP::ValuesConstraint < std::string > formVals ( forms );

		TCLAP::ValueArg < std::string > form ( "f", "form", "Convert to different form", false, "", & formVals );
		cmd.add ( form );

		std::vector < std::string > inputTypes {
			"automaton", "grammar"
		};
		TCLAP::ValuesConstraint < std::string > inputTypeVals ( inputTypes );

		TCLAP::ValueArg < std::string > labels ( "l", "labels", "Normalize labels", false, "", & inputTypeVals );
		cmd.add ( labels );

		std::vector < std::string > inputTypes2 {
			"string"
		};
		TCLAP::ValuesConstraint < std::string > inputTypeVals2 ( inputTypes2 );

		TCLAP::ValueArg < std::string > alphabet ( "a", "alphabet", "Normalize alphabet", false, "", & inputTypeVals2 );
		cmd.add ( alphabet );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if ( verbose.isSet ( ) )
			common::GlobalData::verbose = true;

		if ( measure.isSet ( ) )
			common::GlobalData::measure = true;

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		std::deque < sax::Token > tokens = sax::FromXMLParserHelper::parseInput(input);

		if ( labels.getValue ( ) == "automaton" ) {
			automaton::Automaton automaton = alib::XmlDataFactory::fromTokens < automaton::Automaton > ( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			automaton::Automaton res = automaton::simplify::Normalize::normalize ( automaton );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( labels.getValue ( ) == "grammar" ) {
			// grammar::Grammar grammar = alib::XmlDataFactory::fromTokens<grammar::Grammar>( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			// grammar::Grammar res = grammar::simplify::Normalize::normalize(grammar);

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			// alib::XmlDataFactory::toStdout(res);
		} else if ( alphabet.getValue ( ) == "string" ) {
			string::String string = alib::XmlDataFactory::fromTokens < string::String > ( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			string::String res = string::simplify::NormalizeAlphabet::normalize ( string );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( form.getValue ( ) == "leftRG" ) {
			grammar::Grammar grammar = alib::XmlDataFactory::fromTokens < grammar::Grammar > ( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			grammar::Grammar res = grammar::convert::ToGrammarLeftRG::convert ( grammar );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( form.getValue ( ) == "rightRG" ) {
			grammar::Grammar grammar = alib::XmlDataFactory::fromTokens < grammar::Grammar > ( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			grammar::Grammar res = grammar::convert::ToGrammarRightRG::convert ( grammar );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( form.getValue ( ) == "singleInitialState" ) {
			automaton::Automaton automaton = alib::XmlDataFactory::fromTokens < automaton::Automaton > ( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			automaton::Automaton res = automaton::simplify::SingleInitialState::convert ( automaton );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( form.getValue ( ) == "CNF" ) {
			grammar::Grammar grammar = alib::XmlDataFactory::fromTokens < grammar::Grammar > ( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			grammar::Grammar res = grammar::simplify::ToCNF::convert ( grammar );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( form.getValue ( ) == "GNF" ) {
			grammar::Grammar grammar = alib::XmlDataFactory::fromTokens < grammar::Grammar > ( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			grammar::Grammar res = grammar::simplify::ToGNF::convert ( grammar );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( form.getValue ( ) == "LL1" ) {
			grammar::Grammar grammar = alib::XmlDataFactory::fromTokens < grammar::Grammar > ( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			grammar::Grammar res = grammar::parsing::DeterministicLL1Grammar::convert ( grammar );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else {
			throw exception::CommonException ( "Invalid normalize command" );
		}

		measurements::end ( );
		measurements::end ( );

		if ( measure.getValue ( ) ) std::cmeasure << measurements::results ( ) << std::endl;

		return 0;
	} catch ( const exception::CommonException & exception ) {
		alib::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		std::cout << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		std::cerr << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
