/*
 * astat.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <vector>
#include <sax/FromXMLParserHelper.h>

#include <factory/XmlDataFactory.hpp>
#include <exception/CommonException.h>

#include "SizeStat.h"

template < class T >
void printStat ( const T & data, TCLAP::ValueArg < std::string > & presentation ) {
	if ( presentation.getValue ( ) == "size" )
		SizeStat::stat ( data );
}

int main ( int argc, char * argv[] ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "Prints useful information about raw data structures", ' ', "0.01" );

		// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------

		std::vector < std::string > allowedPrintingOptions {
			"size", "unique_size"
		};
		TCLAP::ValuesConstraint < std::string > allowedPrintingOptionsVals ( allowedPrintingOptions );

		TCLAP::ValueArg < std::string > presentation ( "p", "presentation", "How to print data", true, "noop", & allowedPrintingOptionsVals );
		cmd.add ( presentation );

		// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------

		TCLAP::ValueArg < std::string > file ( "i", "input", "Read from file", false, "-", "file" );
		cmd.add ( file );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if ( verbose.isSet ( ) )
			common::GlobalData::verbose = true;

		if ( measure.isSet ( ) )
			common::GlobalData::measure = true;

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		std::deque < sax::Token > tokens = sax::FromXMLParserHelper::parseInput ( file );

		if ( alib::XmlDataFactory::first < container::Container > ( tokens ) ) {
			container::Container data = alib::XmlDataFactory::fromTokens < container::Container > ( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Stats print", measurements::Type::MAIN );

			printStat ( data, presentation );
		} else if ( alib::XmlDataFactory::first < label::LabelSetLabel > ( tokens ) ) {
			label::LabelSetLabel data = alib::XmlDataFactory::fromTokens < label::LabelSetLabel > ( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Stats print", measurements::Type::MAIN );

			printStat ( data, presentation );
		} else if ( alib::XmlDataFactory::first < alphabet::SymbolSetSymbol > ( tokens ) ) {
			alphabet::SymbolSetSymbol data = alib::XmlDataFactory::fromTokens < alphabet::SymbolSetSymbol > ( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Stats print", measurements::Type::MAIN );

			printStat ( data, presentation );
		} else {
			throw exception::CommonException ( "Input not recognized." );
		}

		measurements::end ( );
		measurements::end ( );

		if ( measure.getValue ( ) ) std::cmeasure << measurements::results ( ) << std::endl;

		return 0;
	} catch ( const exception::CommonException & exception ) {
		alib::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		std::cout << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		std::cerr << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
